"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _express = _interopRequireDefault(require("express"));

var _graphqlRequest = require("graphql-request");

var _pocketJs = require("@pokt-network/pocket-js");

var _environment = _interopRequireDefault(require("../environment"));

var _types = require("../graphql/types");

var _async = _interopRequireDefault(require("../middlewares/async"));

var _passportAuth = require("../middlewares/passport-auth");

var _Application = _interopRequireDefault(require("../models/Application"));

var _PreStakedApp = _interopRequireDefault(require("../models/PreStakedApp"));

var _dateUtils = require("../lib/date-utils");

var _httpError = _interopRequireDefault(require("../errors/http-error"));

var _MailgunService = _interopRequireDefault(require("../services/MailgunService"));

var _pocket = require("../lib/pocket");

var _applicationStatuses = require("../application-statuses");

function _createForOfIteratorHelper(o, allowArrayLike) { var it; if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = o[Symbol.iterator](); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var BUCKETS_PER_HOUR = 2;

var router = _express["default"].Router();

router.use(_passportAuth.authenticate);
router.get('', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2(req, res) {
    var id, application, processedApplications;
    return _regenerator["default"].wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            id = req.user._id;
            _context2.next = 3;
            return _Application["default"].find({
              status: _applicationStatuses.APPLICATION_STATUSES.READY,
              user: id
            });

          case 3:
            application = _context2.sent;

            if (application) {
              _context2.next = 6;
              break;
            }

            throw _httpError["default"].NOT_FOUND({
              errors: [{
                id: 'NONEXISTENT_APPLICATION',
                message: 'User does not have an active application'
              }]
            });

          case 6:
            application.map(function (application) {
              if (application.user.toString() !== id.toString()) {
                throw _httpError["default"].FORBIDDEN({
                  errors: [{
                    id: 'UNAUTHORIZED_ACCESS',
                    message: 'User does not have access to this application'
                  }]
                });
              }
            });
            _context2.next = 9;
            return Promise.all(application.map( /*#__PURE__*/function () {
              var _ref2 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(application) {
                return _regenerator["default"].wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        if (application.updatedAt) {
                          _context.next = 4;
                          break;
                        }

                        application.updatedAt = new Date(Date.now());
                        _context.next = 4;
                        return application.save();

                      case 4:
                      case "end":
                        return _context.stop();
                    }
                  }
                }, _callee);
              }));

              return function (_x3) {
                return _ref2.apply(this, arguments);
              };
            }()));

          case 9:
            processedApplications = application.map(function (app) {
              return {
                apps: [{
                  address: app.freeTierApplicationAccount.address,
                  appId: app._id.toString(),
                  publicKey: app.freeTierApplicationAccount.publicKey
                }],
                chain: app.chain,
                freeTier: app.freeTier,
                gatewaySettings: app.gatewaySettings,
                notificationSettings: app.notificationSettings,
                name: app.name,
                id: app._id.toString(),
                status: app.status,
                updatedAt: app.updatedAt
              };
            });
            res.status(200).send(processedApplications);

          case 11:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function (_x, _x2) {
    return _ref.apply(this, arguments);
  };
}()));
router.get('/:applicationId', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref3 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee3(req, res) {
    var userId, applicationId, application, processedApplication;
    return _regenerator["default"].wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            userId = req.user._id;
            applicationId = req.params.applicationId;
            _context3.next = 4;
            return _Application["default"].findById(applicationId);

          case 4:
            application = _context3.sent;

            if (application) {
              _context3.next = 7;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NONEXISTENT_APPLICATION',
                message: 'User does not have an active application'
              }]
            });

          case 7:
            if (!(application.user.toString() !== userId.toString())) {
              _context3.next = 9;
              break;
            }

            throw _httpError["default"].FORBIDDEN({
              errors: [{
                id: 'UNAUTHORIZED_ACCESS',
                message: 'User does not have access to this application'
              }]
            });

          case 9:
            if (application.updatedAt) {
              _context3.next = 13;
              break;
            }

            application.updatedAt = new Date(Date.now());
            _context3.next = 13;
            return application.save();

          case 13:
            processedApplication = {
              apps: [{
                address: application.freeTierApplicationAccount.address,
                appId: application._id.toString(),
                publicKey: application.freeTierApplicationAccount.publicKey
              }],
              chain: application.chain,
              freeTier: application.freeTier,
              gatewaySettings: application.gatewaySettings,
              name: application.name,
              notificationSettings: application.notificationSettings,
              id: application._id.toString(),
              status: application.status,
              updatedAt: application.updatedAt
            };
            res.status(200).send(processedApplication);

          case 15:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));

  return function (_x4, _x5) {
    return _ref3.apply(this, arguments);
  };
}()));
router.get('/status/:applicationId', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref4 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee4(req, res) {
    var userId, applicationId, application, app, readableApp;
    return _regenerator["default"].wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            userId = req.user._id;
            applicationId = req.params.applicationId;
            _context4.next = 4;
            return _Application["default"].findById(applicationId);

          case 4:
            application = _context4.sent;

            if (application) {
              _context4.next = 7;
              break;
            }

            throw _httpError["default"].NOT_FOUND({
              errors: [{
                id: 'NONEXISTENT_APPLICATION',
                message: 'User does not have an active application'
              }]
            });

          case 7:
            if (!(application.user.toString() !== userId.toString())) {
              _context4.next = 9;
              break;
            }

            throw _httpError["default"].FORBIDDEN({
              errors: [{
                id: 'UNAUTHORIZED_ACCESS',
                message: 'User does not have access to this application'
              }]
            });

          case 9:
            _context4.next = 11;
            return (0, _pocket.getApp)(application.freeTierApplicationAccount.address);

          case 11:
            app = _context4.sent;

            if ((0, _pocketJs.typeGuard)(app, _pocketJs.QueryAppResponse)) {
              _context4.next = 14;
              break;
            }

            throw _httpError["default"].INTERNAL_SERVER_ERROR({
              errors: [{
                id: 'POCKET_JS_ERROR',
                message: 'Application could not be fetched.'
              }]
            });

          case 14:
            readableApp = app.toJSON();
            res.status(200).send({
              stake: readableApp.staked_tokens,
              relays: readableApp.max_relays
            });

          case 16:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4);
  }));

  return function (_x6, _x7) {
    return _ref4.apply(this, arguments);
  };
}()));
router.put('/:applicationId', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref5 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee5(req, res) {
    var gatewaySettings, applicationId, application, userId;
    return _regenerator["default"].wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            gatewaySettings = req.body.gatewaySettings;
            applicationId = req.params.applicationId;
            _context5.next = 4;
            return _Application["default"].findById(applicationId);

          case 4:
            application = _context5.sent;

            if (application) {
              _context5.next = 7;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NONEXISTENT_APPLICATION',
                message: 'Application not found'
              }]
            });

          case 7:
            userId = req.user._id;

            if (!(application.user.toString() !== userId.toString())) {
              _context5.next = 10;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'UNAUTHORIZED_ACCESS',
                message: 'Application does not belong to user'
              }]
            });

          case 10:
            application.gatewaySettings = gatewaySettings;
            _context5.next = 13;
            return application.save();

          case 13:
            // lodash's merge mutates the target object passed in.
            // This is what we want, as we don't want to lose any of the mongoose functionality
            // while at the same time updating the model itself
            res.status(204).send();

          case 14:
          case "end":
            return _context5.stop();
        }
      }
    }, _callee5);
  }));

  return function (_x8, _x9) {
    return _ref5.apply(this, arguments);
  };
}()));
router.post('/switch/:applicationId', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref6 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee6(req, res) {
    var chain, applicationId, replacementApplication, oldApplication, newReplacementApplication, processedApplication;
    return _regenerator["default"].wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            chain = req.body.chain;
            applicationId = req.params.applicationId;
            _context6.next = 4;
            return _PreStakedApp["default"].findOne({
              chain: chain,
              status: _applicationStatuses.APPLICATION_STATUSES.READY
            });

          case 4:
            replacementApplication = _context6.sent;

            if (replacementApplication) {
              _context6.next = 7;
              break;
            }

            throw new Error('No application for the selected chain is available');

          case 7:
            _context6.next = 9;
            return _Application["default"].findById(applicationId);

          case 9:
            oldApplication = _context6.sent;

            if (oldApplication) {
              _context6.next = 12;
              break;
            }

            throw new Error('Cannot find application');

          case 12:
            if (!(oldApplication.user.toString() !== req.user._id.toString())) {
              _context6.next = 14;
              break;
            }

            throw _httpError["default"].FORBIDDEN({
              errors: [{
                id: 'FOREIGN_APPLICATION',
                message: 'Application does not belong to user'
              }]
            });

          case 14:
            oldApplication.status = _applicationStatuses.APPLICATION_STATUSES.AWAITING_GRACE_PERIOD;
            oldApplication.lastChangedStatusAt = Date.now();
            _context6.next = 18;
            return oldApplication.save();

          case 18:
            // Create a new Application for the user and copy the previous user config
            newReplacementApplication = new _Application["default"]({
              // As we're moving to a new chain, everything related to the account and gateway AAT
              // information will change, so we use all the data from the application that we took
              // from the pool.
              chain: replacementApplication.chain,
              freeTierApplicationAccount: replacementApplication.freeTierApplicationAccount,
              gatewayAAT: replacementApplication.gatewayAAT,
              status: _applicationStatuses.APPLICATION_STATUSES.READY,
              lastChangedStatusAt: Date.now(),
              freeTier: true,
              updatedAt: new Date(Date.now()),
              // We wanna preserve user-related configuration fields, so we just copy them over
              // from the old application.
              name: oldApplication.name,
              user: oldApplication.user,
              gatewaySettings: oldApplication.gatewaySettings
            });
            _context6.next = 21;
            return newReplacementApplication.save();

          case 21:
            processedApplication = {
              chain: newReplacementApplication.chain,
              name: newReplacementApplication.name,
              apps: [{
                address: newReplacementApplication.freeTierApplicationAccount.address,
                appId: newReplacementApplication._id.toString(),
                publicKey: newReplacementApplication.freeTierApplicationAccount.publicKey
              }],
              freeTier: newReplacementApplication.freeTier,
              gatewaySettings: newReplacementApplication.gatewaySettings,
              notificationSettings: newReplacementApplication.notificationSettings,
              id: newReplacementApplication._id.toString(),
              status: newReplacementApplication.status,
              updatedAt: newReplacementApplication.updatedAt
            };
            res.status(200).send(processedApplication);

          case 23:
          case "end":
            return _context6.stop();
        }
      }
    }, _callee6);
  }));

  return function (_x10, _x11) {
    return _ref6.apply(this, arguments);
  };
}()));
router.put('/notifications/:applicationId', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref7 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee7(req, res) {
    var applicationId, _req$body, quarter, half, threeQuarters, full, application, emailService, isSignedUp, hasOptedOut;

    return _regenerator["default"].wrap(function _callee7$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:
            applicationId = req.params.applicationId;
            _req$body = req.body, quarter = _req$body.quarter, half = _req$body.half, threeQuarters = _req$body.threeQuarters, full = _req$body.full;
            _context7.next = 4;
            return _Application["default"].findById(applicationId);

          case 4:
            application = _context7.sent;

            if (application) {
              _context7.next = 7;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NONEXISTENT_APPLICATION',
                message: 'This application does not exist'
              }]
            });

          case 7:
            if (!(application.user.toString() !== req.user._id.toString())) {
              _context7.next = 9;
              break;
            }

            throw _httpError["default"].FORBIDDEN({
              errors: [{
                id: 'FOREIGN_APPLICATION',
                message: 'Application does not belong to user'
              }]
            });

          case 9:
            emailService = new _MailgunService["default"]();
            isSignedUp = application.notificationSettings.signedUp;
            hasOptedOut = !(quarter || half || threeQuarters || full);
            application.notificationSettings = {
              signedUp: hasOptedOut ? false : true,
              quarter: quarter,
              half: half,
              threeQuarters: threeQuarters,
              full: full
            };
            _context7.next = 15;
            return application.save();

          case 15:
            if (!isSignedUp) {
              emailService.send({
                templateName: 'NotificationSignup',
                toEmail: req.user.email
              });
            } else {
              emailService.send({
                templateName: 'NotificationChange',
                toEmail: req.user.email
              });
            }

            return _context7.abrupt("return", res.status(204).send());

          case 17:
          case "end":
            return _context7.stop();
        }
      }
    }, _callee7);
  }));

  return function (_x12, _x13) {
    return _ref7.apply(this, arguments);
  };
}()));
router.get('/total-relays/:applicationId', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref8 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee8(req, res) {
    var _result$relay_apps_da, _result$relay_apps_da2;

    var userId, applicationId, application, gqlClient, sevenDaysAgo, result, processedRelaysAndLatency;
    return _regenerator["default"].wrap(function _callee8$(_context8) {
      while (1) {
        switch (_context8.prev = _context8.next) {
          case 0:
            userId = req.user._id;
            applicationId = req.params.applicationId;
            _context8.next = 4;
            return _Application["default"].findById(applicationId);

          case 4:
            application = _context8.sent;

            if (application) {
              _context8.next = 7;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NONEXISTENT_LOADBALANCER',
                message: 'User does not have an active Load Balancer'
              }]
            });

          case 7:
            if (!(application.user.toString() !== userId.toString())) {
              _context8.next = 9;
              break;
            }

            throw _httpError["default"].FORBIDDEN({
              errors: [{
                id: 'UNAUTHORIZED_ACCESS',
                message: 'User does not have access to this load balancer'
              }]
            });

          case 9:
            gqlClient = (0, _types.getSdk)(new _graphqlRequest.GraphQLClient((0, _environment["default"])('HASURA_URL'), {
              // @ts-ignore
              headers: {
                'x-hasura-admin-secret': (0, _environment["default"])('HASURA_SECRET')
              }
            }));
            sevenDaysAgo = (0, _dateUtils.composeDaysFromNowUtcDate)(7);
            _context8.next = 13;
            return gqlClient.getTotalRelaysAndLatency({
              _eq: application.freeTierApplicationAccount.publicKey,
              _gte: sevenDaysAgo
            });

          case 13:
            result = _context8.sent;
            processedRelaysAndLatency = {
              total_relays: (_result$relay_apps_da = result.relay_apps_daily_aggregate.aggregate.sum.total_relays) !== null && _result$relay_apps_da !== void 0 ? _result$relay_apps_da : 0,
              elapsed_time: (_result$relay_apps_da2 = result.relay_apps_daily_aggregate.aggregate.avg.elapsed_time) !== null && _result$relay_apps_da2 !== void 0 ? _result$relay_apps_da2 : 0
            };
            res.status(200).send(processedRelaysAndLatency);

          case 16:
          case "end":
            return _context8.stop();
        }
      }
    }, _callee8);
  }));

  return function (_x14, _x15) {
    return _ref8.apply(this, arguments);
  };
}()));
router.get('/successful-relays/:applicationId', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref9 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee9(req, res) {
    var _result$relay_apps_da3, _result$relay_apps_da4;

    var userId, applicationId, application, gqlClient, sevenDaysAgo, result, processedRelaysAndLatency;
    return _regenerator["default"].wrap(function _callee9$(_context9) {
      while (1) {
        switch (_context9.prev = _context9.next) {
          case 0:
            userId = req.user._id;
            applicationId = req.params.applicationId;
            _context9.next = 4;
            return _Application["default"].findById(applicationId);

          case 4:
            application = _context9.sent;

            if (applicationId) {
              _context9.next = 7;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NONEXISTENT_LOADBALANCER',
                message: 'User does not have an active application'
              }]
            });

          case 7:
            if (!(application.user.toString() !== userId.toString())) {
              _context9.next = 9;
              break;
            }

            throw _httpError["default"].FORBIDDEN({
              errors: [{
                id: 'UNAUTHORIZED_ACCESS',
                message: 'User does not have access to this application'
              }]
            });

          case 9:
            gqlClient = (0, _types.getSdk)(new _graphqlRequest.GraphQLClient((0, _environment["default"])('HASURA_URL'), {
              // @ts-ignore
              headers: {
                'x-hasura-admin-secret': (0, _environment["default"])('HASURA_SECRET')
              }
            }));
            sevenDaysAgo = (0, _dateUtils.composeDaysFromNowUtcDate)(7);
            _context9.next = 13;
            return gqlClient.getTotalSuccessfulRelays({
              _eq: application.freeTierApplicationAccount.publicKey,
              _gte: sevenDaysAgo
            });

          case 13:
            result = _context9.sent;
            processedRelaysAndLatency = {
              total_relays: (_result$relay_apps_da3 = result.relay_apps_daily_aggregate.aggregate.sum.total_relays) !== null && _result$relay_apps_da3 !== void 0 ? _result$relay_apps_da3 : 0,
              elapsed_time: (_result$relay_apps_da4 = result.relay_apps_daily_aggregate.aggregate.avg.elapsed_time) !== null && _result$relay_apps_da4 !== void 0 ? _result$relay_apps_da4 : 0
            };
            res.status(200).send(processedRelaysAndLatency);

          case 16:
          case "end":
            return _context9.stop();
        }
      }
    }, _callee9);
  }));

  return function (_x16, _x17) {
    return _ref9.apply(this, arguments);
  };
}()));
router.get('/daily-relays/:applicationId', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref10 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee10(req, res) {
    var userId, applicationId, application, gqlClient, sevenDaysAgo, result, dailyRelays, _iterator, _step, _step$value, bucket, dailyRelayCount, currentCount, processedDailyRelays, _iterator2, _step2, _step2$value, _bucket, _dailyRelayCount;

    return _regenerator["default"].wrap(function _callee10$(_context10) {
      while (1) {
        switch (_context10.prev = _context10.next) {
          case 0:
            userId = req.user._id;
            applicationId = req.params.applicationId;
            _context10.next = 4;
            return _Application["default"].findById(applicationId);

          case 4:
            application = _context10.sent;

            if (applicationId) {
              _context10.next = 7;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NONEXISTENT_LOADBALANCER',
                message: 'User does not have an active application'
              }]
            });

          case 7:
            if (!(application.user.toString() !== userId.toString())) {
              _context10.next = 9;
              break;
            }

            throw _httpError["default"].FORBIDDEN({
              errors: [{
                id: 'UNAUTHORIZED_ACCESS',
                message: 'User does not have access to this application'
              }]
            });

          case 9:
            gqlClient = (0, _types.getSdk)(new _graphqlRequest.GraphQLClient((0, _environment["default"])('HASURA_URL'), {
              // @ts-ignore
              headers: {
                'x-hasura-admin-secret': (0, _environment["default"])('HASURA_SECRET')
              }
            }));
            sevenDaysAgo = (0, _dateUtils.composeDaysFromNowUtcDate)(7);
            _context10.next = 13;
            return gqlClient.getDailyTotalRelays({
              _eq: application.freeTierApplicationAccount.publicKey,
              _gte: sevenDaysAgo
            });

          case 13:
            result = _context10.sent;
            dailyRelays = new Map();
            _iterator = _createForOfIteratorHelper(result.relay_apps_daily);

            try {
              for (_iterator.s(); !(_step = _iterator.n()).done;) {
                _step$value = _step.value, bucket = _step$value.bucket, dailyRelayCount = _step$value.total_relays;

                if (!dailyRelays.has(bucket)) {
                  dailyRelays.set(bucket, dailyRelayCount !== null && dailyRelayCount !== void 0 ? dailyRelayCount : 0);
                } else {
                  currentCount = dailyRelays.get(bucket);
                  dailyRelays.set(bucket, Number(currentCount) + Number(dailyRelayCount));
                }
              }
            } catch (err) {
              _iterator.e(err);
            } finally {
              _iterator.f();
            }

            processedDailyRelays = [];
            _iterator2 = _createForOfIteratorHelper(dailyRelays.entries());

            try {
              for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
                _step2$value = (0, _slicedToArray2["default"])(_step2.value, 2), _bucket = _step2$value[0], _dailyRelayCount = _step2$value[1];
                processedDailyRelays.push({
                  bucket: _bucket,
                  dailyRelays: _dailyRelayCount
                });
              }
            } catch (err) {
              _iterator2.e(err);
            } finally {
              _iterator2.f();
            }

            res.status(200).send({
              daily_relays: processedDailyRelays.reverse()
            });

          case 21:
          case "end":
            return _context10.stop();
        }
      }
    }, _callee10);
  }));

  return function (_x18, _x19) {
    return _ref10.apply(this, arguments);
  };
}()));
router.get('/session-relays/:applicationId', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref11 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee11(req, res) {
    var userId, applicationId, application, gqlClient, today, result, totalSessionRelays;
    return _regenerator["default"].wrap(function _callee11$(_context11) {
      while (1) {
        switch (_context11.prev = _context11.next) {
          case 0:
            userId = req.user._id;
            applicationId = req.params.applicationId;
            _context11.next = 4;
            return _Application["default"].findById(applicationId);

          case 4:
            application = _context11.sent;

            if (applicationId) {
              _context11.next = 7;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NONEXISTENT_LOADBALANCER',
                message: 'User does not have an active application'
              }]
            });

          case 7:
            if (!(application.user.toString() !== userId.toString())) {
              _context11.next = 9;
              break;
            }

            throw _httpError["default"].FORBIDDEN({
              errors: [{
                id: 'UNAUTHORIZED_ACCESS',
                message: 'User does not have access to this application'
              }]
            });

          case 9:
            gqlClient = (0, _types.getSdk)(new _graphqlRequest.GraphQLClient((0, _environment["default"])('HASURA_URL'), {
              // @ts-ignore
              headers: {
                'x-hasura-admin-secret': (0, _environment["default"])('HASURA_SECRET')
              }
            }));
            today = (0, _dateUtils.composeTodayUtcDate)();
            _context11.next = 13;
            return gqlClient.getLastSessionAppRelays({
              _eq: application.freeTierApplicationAccount.publicKey,
              _gte: today,
              _buckets: BUCKETS_PER_HOUR
            });

          case 13:
            result = _context11.sent;
            totalSessionRelays = result.relay_app_hourly.reduce(function (total, _ref12) {
              var totalRelays = _ref12.total_relays;
              return total + totalRelays;
            }, 0);
            res.status(200).send({
              session_relays: totalSessionRelays
            });

          case 16:
          case "end":
            return _context11.stop();
        }
      }
    }, _callee11);
  }));

  return function (_x20, _x21) {
    return _ref11.apply(this, arguments);
  };
}()));
router.post('/latest-relays', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref13 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee12(req, res) {
    var userId, _req$body2, id, limit, offset, application, gqlClient, result, relays;

    return _regenerator["default"].wrap(function _callee12$(_context12) {
      while (1) {
        switch (_context12.prev = _context12.next) {
          case 0:
            userId = req.user._id;
            _req$body2 = req.body, id = _req$body2.id, limit = _req$body2.limit, offset = _req$body2.offset;
            _context12.next = 4;
            return _Application["default"].findById(id);

          case 4:
            application = _context12.sent;

            if (id) {
              _context12.next = 7;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NONEXISTENT_LOADBALANCER',
                message: 'User does not have an active application'
              }]
            });

          case 7:
            if (!(application.user.toString() !== userId.toString())) {
              _context12.next = 9;
              break;
            }

            throw _httpError["default"].FORBIDDEN({
              errors: [{
                id: 'UNAUTHORIZED_ACCESS',
                message: 'User does not have access to this application'
              }]
            });

          case 9:
            gqlClient = (0, _types.getSdk)(new _graphqlRequest.GraphQLClient((0, _environment["default"])('HASURA_URL'), {
              // @ts-ignore
              headers: {
                'x-hasura-admin-secret': (0, _environment["default"])('HASURA_SECRET')
              }
            }));
            _context12.next = 12;
            return gqlClient.getLatestRelays({
              _eq: application.freeTierApplicationAccount.publicKey,
              limit: limit,
              offset: offset
            });

          case 12:
            result = _context12.sent;
            relays = result.relay;
            relays.sort(function (a, b) {
              var dateA = new Date(a.timestamp);
              var dateB = new Date(b.timestamp); // @ts-ignore

              return dateA - dateB;
            }).reverse();
            res.status(200).send({
              session_relays: relays.slice(0, 10)
            });

          case 16:
          case "end":
            return _context12.stop();
        }
      }
    }, _callee12);
  }));

  return function (_x22, _x23) {
    return _ref13.apply(this, arguments);
  };
}()));
router.post('/latest-successful-relays', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref14 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee13(req, res) {
    var userId, _req$body3, id, limit, offset, application, gqlClient, latestSuccessfulRelays, relays;

    return _regenerator["default"].wrap(function _callee13$(_context13) {
      while (1) {
        switch (_context13.prev = _context13.next) {
          case 0:
            userId = req.user._id;
            _req$body3 = req.body, id = _req$body3.id, limit = _req$body3.limit, offset = _req$body3.offset;
            _context13.next = 4;
            return _Application["default"].findById(id);

          case 4:
            application = _context13.sent;

            if (application) {
              _context13.next = 7;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NONEXISTENT_LOADBALANCER',
                message: 'User does not have an active application'
              }]
            });

          case 7:
            if (!(application.user.toString() !== userId.toString())) {
              _context13.next = 9;
              break;
            }

            throw _httpError["default"].FORBIDDEN({
              errors: [{
                id: 'UNAUTHORIZED_ACCESS',
                message: 'User does not have access to this load balancer'
              }]
            });

          case 9:
            gqlClient = (0, _types.getSdk)(new _graphqlRequest.GraphQLClient((0, _environment["default"])('HASURA_URL'), {
              // @ts-ignore
              headers: {
                'x-hasura-admin-secret': (0, _environment["default"])('HASURA_SECRET')
              }
            }));
            _context13.next = 12;
            return gqlClient.getLatestSuccessfulRelays({
              _eq: application.freeTierApplicationAccount.publicKey,
              _eq1: 200,
              offset: offset
            });

          case 12:
            latestSuccessfulRelays = _context13.sent;
            relays = [];
            latestSuccessfulRelays.relay.map(function (relayBatch) {
              relays.push(relayBatch);
            });
            relays.sort(function (a, b) {
              var dateA = new Date(a.timestamp);
              var dateB = new Date(b.timestamp); // @ts-ignore

              return dateA - dateB;
            }).reverse();
            res.status(200).send({
              session_relays: relays.slice(0, 10)
            });

          case 17:
          case "end":
            return _context13.stop();
        }
      }
    }, _callee13);
  }));

  return function (_x24, _x25) {
    return _ref14.apply(this, arguments);
  };
}()));
router.post('/latest-failing-relays', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref15 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee14(req, res) {
    var userId, _req$body4, id, offset, application, gqlClient, latestSuccessfulRelays, relays;

    return _regenerator["default"].wrap(function _callee14$(_context14) {
      while (1) {
        switch (_context14.prev = _context14.next) {
          case 0:
            userId = req.user._id;
            _req$body4 = req.body, id = _req$body4.id, offset = _req$body4.offset;
            _context14.next = 4;
            return _Application["default"].findById(id);

          case 4:
            application = _context14.sent;

            if (application) {
              _context14.next = 7;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NONEXISTENT_LOADBALANCER',
                message: 'User does not have an active application'
              }]
            });

          case 7:
            if (!(application.user.toString() !== userId.toString())) {
              _context14.next = 9;
              break;
            }

            throw _httpError["default"].FORBIDDEN({
              errors: [{
                id: 'UNAUTHORIZED_ACCESS',
                message: 'User does not have access to this load balancer'
              }]
            });

          case 9:
            gqlClient = (0, _types.getSdk)(new _graphqlRequest.GraphQLClient((0, _environment["default"])('HASURA_URL'), {
              // @ts-ignore
              headers: {
                'x-hasura-admin-secret': (0, _environment["default"])('HASURA_SECRET')
              }
            }));
            _context14.next = 12;
            return gqlClient.getLatestFailingRelays({
              _eq: application.freeTierApplicationAccount.publicKey,
              _eq1: 200,
              offset: offset
            });

          case 12:
            latestSuccessfulRelays = _context14.sent;
            relays = [];
            latestSuccessfulRelays.relay.map(function (relayBatch) {
              relays.push(relayBatch);
            });
            relays.sort(function (a, b) {
              var dateA = new Date(a.timestamp);
              var dateB = new Date(b.timestamp); // @ts-ignore

              return dateA - dateB;
            }).reverse();
            res.status(200).send({
              session_relays: relays.slice(0, 10)
            });

          case 17:
          case "end":
            return _context14.stop();
        }
      }
    }, _callee14);
  }));

  return function (_x26, _x27) {
    return _ref15.apply(this, arguments);
  };
}()));
router.get('/ranged-relays/:applicationId', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref16 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee15(req, res) {
    var _result$relay_apps_da5;

    var userId, applicationId, application, gqlClient, fourteenDaysAgo, sevenDaysAgo, result, totalRangedRelays;
    return _regenerator["default"].wrap(function _callee15$(_context15) {
      while (1) {
        switch (_context15.prev = _context15.next) {
          case 0:
            userId = req.user._id;
            applicationId = req.params.applicationId;
            _context15.next = 4;
            return _Application["default"].findById(applicationId);

          case 4:
            application = _context15.sent;

            if (applicationId) {
              _context15.next = 7;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NONEXISTENT_LOADBALANCER',
                message: 'User does not have an active application'
              }]
            });

          case 7:
            if (!(application.user.toString() !== userId.toString())) {
              _context15.next = 9;
              break;
            }

            throw _httpError["default"].FORBIDDEN({
              errors: [{
                id: 'UNAUTHORIZED_ACCESS',
                message: 'User does not have access to this application'
              }]
            });

          case 9:
            gqlClient = (0, _types.getSdk)(new _graphqlRequest.GraphQLClient((0, _environment["default"])('HASURA_URL'), {
              // @ts-ignore
              headers: {
                'x-hasura-admin-secret': (0, _environment["default"])('HASURA_SECRET')
              }
            }));
            fourteenDaysAgo = (0, _dateUtils.composeDaysFromNowUtcDate)(14);
            sevenDaysAgo = (0, _dateUtils.composeDaysFromNowUtcDate)(7);
            _context15.next = 14;
            return gqlClient.getTotalRangedRelaysAndLatency({
              _eq: application.freeTierApplicationAccount.publicKey,
              _gte: fourteenDaysAgo,
              _lte: sevenDaysAgo
            });

          case 14:
            result = _context15.sent;
            totalRangedRelays = {
              total_relays: (_result$relay_apps_da5 = result.relay_apps_daily_aggregate.aggregate.sum.total_relays) !== null && _result$relay_apps_da5 !== void 0 ? _result$relay_apps_da5 : 0
            };
            res.status(200).send({
              total_relays: totalRangedRelays.total_relays
            });

          case 17:
          case "end":
            return _context15.stop();
        }
      }
    }, _callee15);
  }));

  return function (_x28, _x29) {
    return _ref16.apply(this, arguments);
  };
}()));
router.get('/previous-successful-relays/:applicationId', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref17 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee16(req, res) {
    var _result$relay_apps_da6;

    var userId, applicationId, application, gqlClient, fourteenDaysAgo, sevenDaysAgo, result, totalSuccessfulRelays;
    return _regenerator["default"].wrap(function _callee16$(_context16) {
      while (1) {
        switch (_context16.prev = _context16.next) {
          case 0:
            userId = req.user._id;
            applicationId = req.params.applicationId;
            _context16.next = 4;
            return _Application["default"].findById(applicationId);

          case 4:
            application = _context16.sent;

            if (applicationId) {
              _context16.next = 7;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NONEXISTENT_LOADBALANCER',
                message: 'User does not have an active application'
              }]
            });

          case 7:
            if (!(application.user.toString() !== userId.toString())) {
              _context16.next = 9;
              break;
            }

            throw _httpError["default"].FORBIDDEN({
              errors: [{
                id: 'UNAUTHORIZED_ACCESS',
                message: 'User does not have access to this application'
              }]
            });

          case 9:
            gqlClient = (0, _types.getSdk)(new _graphqlRequest.GraphQLClient((0, _environment["default"])('HASURA_URL'), {
              // @ts-ignore
              headers: {
                'x-hasura-admin-secret': (0, _environment["default"])('HASURA_SECRET')
              }
            }));
            fourteenDaysAgo = (0, _dateUtils.composeDaysFromNowUtcDate)(14);
            sevenDaysAgo = (0, _dateUtils.composeDaysFromNowUtcDate)(7);
            _context16.next = 14;
            return gqlClient.getTotalSuccessfulRangedRelays({
              _eq: application.freeTierApplicationAccount.publicKey,
              _gte: fourteenDaysAgo,
              _lte: sevenDaysAgo
            });

          case 14:
            result = _context16.sent;
            totalSuccessfulRelays = {
              successful_relays: (_result$relay_apps_da6 = result.relay_apps_daily_aggregate.aggregate.sum.total_relays) !== null && _result$relay_apps_da6 !== void 0 ? _result$relay_apps_da6 : 0
            };
            res.status(200).send({
              successful_relays: totalSuccessfulRelays.successful_relays
            });

          case 17:
          case "end":
            return _context16.stop();
        }
      }
    }, _callee16);
  }));

  return function (_x30, _x31) {
    return _ref17.apply(this, arguments);
  };
}()));
router.get('/hourly-latency/:applicationId', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref18 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee17(req, res) {
    var userId, applicationId, application, gqlClient, aDayAgo, hourlyLatency, result, _iterator3, _step3, _step3$value, bucket, elapsedTime, currentCount, processedHourlyLatency, _iterator4, _step4, _step4$value, _bucket2, hourlyLatencyAvg;

    return _regenerator["default"].wrap(function _callee17$(_context17) {
      while (1) {
        switch (_context17.prev = _context17.next) {
          case 0:
            userId = req.user._id;
            applicationId = req.params.applicationId;
            _context17.next = 4;
            return _Application["default"].findById(applicationId);

          case 4:
            application = _context17.sent;

            if (applicationId) {
              _context17.next = 7;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NONEXISTENT_LOADBALANCER',
                message: 'User does not have an active application'
              }]
            });

          case 7:
            if (!(application.user.toString() !== userId.toString())) {
              _context17.next = 9;
              break;
            }

            throw _httpError["default"].FORBIDDEN({
              errors: [{
                id: 'UNAUTHORIZED_ACCESS',
                message: 'User does not have access to this application'
              }]
            });

          case 9:
            gqlClient = (0, _types.getSdk)(new _graphqlRequest.GraphQLClient((0, _environment["default"])('HASURA_URL'), {
              // @ts-ignore
              headers: {
                'x-hasura-admin-secret': (0, _environment["default"])('HASURA_SECRET')
              }
            }));
            aDayAgo = (0, _dateUtils.composeHoursFromNowUtcDate)(24);
            hourlyLatency = new Map();
            _context17.next = 14;
            return gqlClient.getTotalRelayDuration({
              _eq: application.freeTierApplicationAccount.publicKey,
              _gte: aDayAgo
            });

          case 14:
            result = _context17.sent;
            _iterator3 = _createForOfIteratorHelper(result.relay_app_hourly);

            try {
              for (_iterator3.s(); !(_step3 = _iterator3.n()).done;) {
                _step3$value = _step3.value, bucket = _step3$value.bucket, elapsedTime = _step3$value.elapsed_time;

                if (!hourlyLatency.has(bucket)) {
                  hourlyLatency.set(bucket, elapsedTime !== null && elapsedTime !== void 0 ? elapsedTime : 0);
                } else {
                  currentCount = hourlyLatency.get(bucket);
                  hourlyLatency.set(bucket, (Number(currentCount) + Number(elapsedTime)) / 2);
                }
              }
            } catch (err) {
              _iterator3.e(err);
            } finally {
              _iterator3.f();
            }

            processedHourlyLatency = [];
            _iterator4 = _createForOfIteratorHelper(hourlyLatency.entries());

            try {
              for (_iterator4.s(); !(_step4 = _iterator4.n()).done;) {
                _step4$value = (0, _slicedToArray2["default"])(_step4.value, 2), _bucket2 = _step4$value[0], hourlyLatencyAvg = _step4$value[1];
                processedHourlyLatency.push({
                  bucket: _bucket2,
                  latency: hourlyLatencyAvg
                });
              }
            } catch (err) {
              _iterator4.e(err);
            } finally {
              _iterator4.f();
            }

            res.status(200).send({
              hourly_latency: processedHourlyLatency.reverse()
            });

          case 21:
          case "end":
            return _context17.stop();
        }
      }
    }, _callee17);
  }));

  return function (_x32, _x33) {
    return _ref18.apply(this, arguments);
  };
}()));
var _default = router;
exports["default"] = _default;
//# sourceMappingURL=ApplicationController.js.map