"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _express = _interopRequireDefault(require("express"));

var _crypto = _interopRequireDefault(require("crypto"));

var _graphqlRequest = require("graphql-request");

var _pocketJs = require("@pokt-network/pocket-js");

var _environment = _interopRequireDefault(require("../environment"));

var _types = require("../graphql/types");

var _async = _interopRequireDefault(require("../middlewares/async"));

var _passportAuth = require("../middlewares/passport-auth");

var _Application = _interopRequireDefault(require("../models/Application"));

var _PreStakedApp = _interopRequireDefault(require("../models/PreStakedApp"));

var _LoadBalancer = _interopRequireDefault(require("../models/LoadBalancer"));

var _dateUtils = require("../lib/date-utils");

var _pocket = require("../lib/pocket");

var _httpError = _interopRequireDefault(require("../errors/http-error"));

var _MailgunService = _interopRequireDefault(require("../services/MailgunService"));

var _applicationStatuses = require("../application-statuses");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _createForOfIteratorHelper(o, allowArrayLike) { var it; if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = o[Symbol.iterator](); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var DEFAULT_GATEWAY_SETTINGS = {
  secretKey: '',
  secretKeyRequired: false,
  whitelistOrigins: [],
  whitelistUserAgents: []
};
var DEFAULT_TIMEOUT = 2000;
var BUCKETS_PER_HOUR = 2;

var router = _express["default"].Router();

router.use(_passportAuth.authenticate);
router.get('', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2(req, res) {
    var id, lbs, processedLbs;
    return _regenerator["default"].wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            id = req.user._id;
            _context2.next = 3;
            return _LoadBalancer["default"].find({
              user: id
            });

          case 3:
            lbs = _context2.sent;

            if (lbs) {
              _context2.next = 6;
              break;
            }

            throw _httpError["default"].NOT_FOUND({
              errors: [{
                id: 'NONEXISTENT_APPLICATION',
                message: 'User does not have an active application'
              }]
            });

          case 6:
            _context2.next = 8;
            return Promise.all(lbs.map( /*#__PURE__*/function () {
              var _ref2 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(lb) {
                var apps, _iterator, _step, appId, _app, app, processedLb;

                return _regenerator["default"].wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        if (lb.applicationIDs.length) {
                          _context.next = 2;
                          break;
                        }

                        throw _httpError["default"].INTERNAL_SERVER_ERROR({
                          errors: [{
                            id: 'MALFORMED_LB',
                            message: 'Malformed load balancer'
                          }]
                        });

                      case 2:
                        if (lb.updatedAt) {
                          _context.next = 6;
                          break;
                        }

                        lb.updatedAt = new Date(Date.now());
                        _context.next = 6;
                        return lb.save();

                      case 6:
                        apps = [];
                        _iterator = _createForOfIteratorHelper(lb.applicationIDs);
                        _context.prev = 8;

                        _iterator.s();

                      case 10:
                        if ((_step = _iterator.n()).done) {
                          _context.next = 18;
                          break;
                        }

                        appId = _step.value;
                        _context.next = 14;
                        return _Application["default"].findById(appId);

                      case 14:
                        _app = _context.sent;
                        apps.push({
                          appId: _app._id.toString(),
                          address: _app.freeTierApplicationAccount.address,
                          publicKey: _app.freeTierApplicationAccount.publicKey
                        });

                      case 16:
                        _context.next = 10;
                        break;

                      case 18:
                        _context.next = 23;
                        break;

                      case 20:
                        _context.prev = 20;
                        _context.t0 = _context["catch"](8);

                        _iterator.e(_context.t0);

                      case 23:
                        _context.prev = 23;

                        _iterator.f();

                        return _context.finish(23);

                      case 26:
                        _context.next = 28;
                        return _Application["default"].findById(lb.applicationIDs[0]);

                      case 28:
                        app = _context.sent;
                        processedLb = {
                          apps: apps,
                          chain: app.chain,
                          createdAt: new Date(Date.now()),
                          updatedAt: lb.updatedAt,
                          freeTier: app.freeTier,
                          gatewaySettings: app.gatewaySettings,
                          notificationSettings: app.notificationSettings,
                          name: lb.name,
                          id: lb._id.toString(),
                          status: app.status
                        };
                        return _context.abrupt("return", processedLb);

                      case 31:
                      case "end":
                        return _context.stop();
                    }
                  }
                }, _callee, null, [[8, 20, 23, 26]]);
              }));

              return function (_x3) {
                return _ref2.apply(this, arguments);
              };
            }()));

          case 8:
            processedLbs = _context2.sent;
            res.status(200).send(processedLbs);

          case 10:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function (_x, _x2) {
    return _ref.apply(this, arguments);
  };
}()));
router.post('', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref3 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee3(req, res) {
    var _req$body, name, chain, _req$body$gatewaySett, gatewaySettings, id, isNewAppRequestInvalid, preStakedApp, application, _yield$ApplicationPoo, ok, loadBalancer, processedLb;

    return _regenerator["default"].wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _req$body = req.body, name = _req$body.name, chain = _req$body.chain, _req$body$gatewaySett = _req$body.gatewaySettings, gatewaySettings = _req$body$gatewaySett === void 0 ? DEFAULT_GATEWAY_SETTINGS : _req$body$gatewaySett;
            id = req.user._id;
            _context3.next = 4;
            return _Application["default"].exists({
              status: _applicationStatuses.APPLICATION_STATUSES.READY,
              user: id
            });

          case 4:
            _context3.t0 = _context3.sent;

            if (_context3.t0) {
              _context3.next = 9;
              break;
            }

            _context3.next = 8;
            return _LoadBalancer["default"].exists({
              user: id
            });

          case 8:
            _context3.t0 = _context3.sent;

          case 9:
            isNewAppRequestInvalid = _context3.t0;

            if (!isNewAppRequestInvalid) {
              _context3.next = 12;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'ALREADY_EXISTING',
                message: 'User already has an existing free tier app'
              }]
            });

          case 12:
            _context3.next = 14;
            return _PreStakedApp["default"].findOne({
              status: _applicationStatuses.APPLICATION_STATUSES.READY,
              chain: chain
            });

          case 14:
            preStakedApp = _context3.sent;

            if (preStakedApp) {
              _context3.next = 17;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'POOL_EMPTY',
                message: 'No pre-staked apps available for this chain.'
              }]
            });

          case 17:
            application = new _Application["default"]({
              chain: chain,
              name: name,
              user: id,
              status: _applicationStatuses.APPLICATION_STATUSES.READY,
              lastChangedStatusAt: new Date(Date.now()),
              // We enforce every app to be treated as a free-tier app for now.
              freeTier: true,
              freeTierApplicationAccount: preStakedApp.freeTierApplicationAccount,
              gatewayAAT: preStakedApp.gatewayAAT,
              gatewaySettings: _objectSpread({}, gatewaySettings),
              notificationSettings: {
                signedUp: false,
                quarter: false,
                half: false,
                threeQuarters: false,
                full: false
              }
            });
            application.gatewaySettings.secretKey = _crypto["default"].randomBytes(16).toString('hex');
            _context3.next = 21;
            return application.save();

          case 21:
            _context3.next = 23;
            return _PreStakedApp["default"].deleteOne({
              _id: preStakedApp._id
            });

          case 23:
            _yield$ApplicationPoo = _context3.sent;
            ok = _yield$ApplicationPoo.ok;

            if (!(ok !== 1)) {
              _context3.next = 27;
              break;
            }

            throw _httpError["default"].INTERNAL_SERVER_ERROR({
              errors: [{
                id: 'DB_ERROR',
                message: 'There was an error while updating the DB'
              }]
            });

          case 27:
            loadBalancer = new _LoadBalancer["default"]({
              user: id,
              name: name,
              requestTimeOut: DEFAULT_TIMEOUT,
              applicationIDs: [application._id.toString()],
              updatedAt: new Date(Date.now())
            });
            _context3.next = 30;
            return loadBalancer.save();

          case 30:
            processedLb = {
              chain: application.chain,
              createdAt: new Date(Date.now()),
              updatedAt: loadBalancer.updatedAt,
              name: loadBalancer.name,
              id: loadBalancer._id.toString(),
              freeTier: true,
              status: application.status,
              apps: [{
                appId: application._id.toString(),
                address: application.freeTierApplicationAccount.address,
                publicKey: application.freeTierApplicationAccount.publicKey
              }],
              gatewaySettings: application.gatewaySettings,
              notificationSettings: application.notificationSettings
            };
            res.status(200).send(processedLb);

          case 32:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));

  return function (_x4, _x5) {
    return _ref3.apply(this, arguments);
  };
}()));
router.put('/:lbId', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref4 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee6(req, res) {
    var gatewaySettings, lbId, userId, loadBalancer, existingSettings, _existingSettings$fin, _existingSettings$fin2, secretKey;

    return _regenerator["default"].wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            gatewaySettings = req.body.gatewaySettings;
            lbId = req.params.lbId;
            userId = req.user._id;
            _context6.next = 5;
            return _LoadBalancer["default"].findById(lbId);

          case 5:
            loadBalancer = _context6.sent;

            if (loadBalancer) {
              _context6.next = 8;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NONEXISTENT_APPLICATION',
                message: 'Application not found'
              }]
            });

          case 8:
            if (!(loadBalancer.user.toString() !== userId.toString())) {
              _context6.next = 10;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'UNAUTHORIZED_ACCESS',
                message: 'Application does not belong to user'
              }]
            });

          case 10:
            _context6.next = 12;
            return Promise.all(loadBalancer.applicationIDs.map( /*#__PURE__*/function () {
              var _changeSettings = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee4(applicationId) {
                var application;
                return _regenerator["default"].wrap(function _callee4$(_context4) {
                  while (1) {
                    switch (_context4.prev = _context4.next) {
                      case 0:
                        _context4.next = 2;
                        return _Application["default"].findById(applicationId);

                      case 2:
                        application = _context4.sent;
                        return _context4.abrupt("return", application.gatewaySettings);

                      case 4:
                      case "end":
                        return _context4.stop();
                    }
                  }
                }, _callee4);
              }));

              function changeSettings(_x8) {
                return _changeSettings.apply(this, arguments);
              }

              return changeSettings;
            }()));

          case 12:
            existingSettings = _context6.sent;
            _existingSettings$fin = existingSettings.find(function (settings) {
              return (settings === null || settings === void 0 ? void 0 : settings.secretKey) !== '';
            }), _existingSettings$fin2 = _existingSettings$fin.secretKey, secretKey = _existingSettings$fin2 === void 0 ? '' : _existingSettings$fin2;
            gatewaySettings.secretKey = secretKey;
            _context6.next = 17;
            return Promise.all(loadBalancer.applicationIDs.map( /*#__PURE__*/function () {
              var _changeSettings2 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee5(applicationId) {
                var application;
                return _regenerator["default"].wrap(function _callee5$(_context5) {
                  while (1) {
                    switch (_context5.prev = _context5.next) {
                      case 0:
                        _context5.next = 2;
                        return _Application["default"].findById(applicationId);

                      case 2:
                        application = _context5.sent;
                        application.gatewaySettings = gatewaySettings;
                        _context5.next = 6;
                        return application.save();

                      case 6:
                      case "end":
                        return _context5.stop();
                    }
                  }
                }, _callee5);
              }));

              function changeSettings(_x9) {
                return _changeSettings2.apply(this, arguments);
              }

              return changeSettings;
            }()));

          case 17:
            res.status(204).send();

          case 18:
          case "end":
            return _context6.stop();
        }
      }
    }, _callee6);
  }));

  return function (_x6, _x7) {
    return _ref4.apply(this, arguments);
  };
}()));
router.get('/status/:lbId', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref5 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee8(req, res) {
    var userId, lbId, loadBalancer, apps, isAppResponseOk, readableApps, appsStatus;
    return _regenerator["default"].wrap(function _callee8$(_context8) {
      while (1) {
        switch (_context8.prev = _context8.next) {
          case 0:
            userId = req.user._id;
            lbId = req.params.lbId;
            _context8.next = 4;
            return _LoadBalancer["default"].findById(lbId);

          case 4:
            loadBalancer = _context8.sent;

            if (loadBalancer) {
              _context8.next = 7;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NONEXISTENT_APPLICATION',
                message: 'Application not found'
              }]
            });

          case 7:
            if (!(loadBalancer.user.toString() !== userId.toString())) {
              _context8.next = 9;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'UNAUTHORIZED_ACCESS',
                message: 'Application does not belong to user'
              }]
            });

          case 9:
            _context8.next = 11;
            return Promise.all(loadBalancer.applicationIDs.map( /*#__PURE__*/function () {
              var _getApps = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee7(applicationId) {
                var application;
                return _regenerator["default"].wrap(function _callee7$(_context7) {
                  while (1) {
                    switch (_context7.prev = _context7.next) {
                      case 0:
                        _context7.next = 2;
                        return _Application["default"].findById(applicationId);

                      case 2:
                        application = _context7.sent;
                        _context7.next = 5;
                        return (0, _pocket.getApp)(application.freeTierApplicationAccount.address);

                      case 5:
                        return _context7.abrupt("return", _context7.sent);

                      case 6:
                      case "end":
                        return _context7.stop();
                    }
                  }
                }, _callee7);
              }));

              function getApps(_x12) {
                return _getApps.apply(this, arguments);
              }

              return getApps;
            }()));

          case 11:
            apps = _context8.sent;
            isAppResponseOk = apps.reduce(function (ok, app) {
              return ok || (0, _pocketJs.typeGuard)(app, _pocketJs.QueryAppResponse);
            }, true);

            if (isAppResponseOk) {
              _context8.next = 15;
              break;
            }

            throw _httpError["default"].INTERNAL_SERVER_ERROR({
              errors: [{
                id: 'POCKET_JS_ERROR',
                message: 'Application could not be fetched.'
              }]
            });

          case 15:
            readableApps = apps.map(function (app) {
              return app.toJSON();
            });
            appsStatus = readableApps.reduce(function (status, app) {
              return {
                stake: app.staked_tokens + status.stake,
                relays: app.max_relays + status.relays
              };
            }, {
              stake: 0,
              relays: 0
            });
            res.status(200).send(appsStatus);

          case 18:
          case "end":
            return _context8.stop();
        }
      }
    }, _callee8);
  }));

  return function (_x10, _x11) {
    return _ref5.apply(this, arguments);
  };
}()));
router.put('/notifications/:lbId', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref6 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee10(req, res) {
    var userId, lbId, _req$body2, quarter, half, threeQuarters, full, loadBalancer, emailService, hasOptedOut, notificationSettings;

    return _regenerator["default"].wrap(function _callee10$(_context10) {
      while (1) {
        switch (_context10.prev = _context10.next) {
          case 0:
            userId = req.user._id;
            lbId = req.params.lbId;
            _req$body2 = req.body, quarter = _req$body2.quarter, half = _req$body2.half, threeQuarters = _req$body2.threeQuarters, full = _req$body2.full;
            _context10.next = 5;
            return _LoadBalancer["default"].findById(lbId);

          case 5:
            loadBalancer = _context10.sent;

            if (loadBalancer) {
              _context10.next = 8;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NONEXISTENT_APPLICATION',
                message: 'Application not found'
              }]
            });

          case 8:
            if (!(loadBalancer.user.toString() !== userId.toString())) {
              _context10.next = 10;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'UNAUTHORIZED_ACCESS',
                message: 'Application does not belong to user'
              }]
            });

          case 10:
            emailService = new _MailgunService["default"]();
            hasOptedOut = !(quarter || half || threeQuarters || full);
            notificationSettings = {
              signedUp: hasOptedOut ? false : true,
              quarter: quarter,
              half: half,
              threeQuarters: threeQuarters,
              full: full
            };
            _context10.next = 15;
            return Promise.all(loadBalancer.applicationIDs.map( /*#__PURE__*/function () {
              var _ref7 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee9(id) {
                var app;
                return _regenerator["default"].wrap(function _callee9$(_context9) {
                  while (1) {
                    switch (_context9.prev = _context9.next) {
                      case 0:
                        _context9.next = 2;
                        return _Application["default"].findById(id);

                      case 2:
                        app = _context9.sent;
                        app.notificationSettings = notificationSettings;
                        _context9.next = 6;
                        return app.save();

                      case 6:
                      case "end":
                        return _context9.stop();
                    }
                  }
                }, _callee9);
              }));

              return function (_x15) {
                return _ref7.apply(this, arguments);
              };
            }()));

          case 15:
            emailService.send({
              templateName: 'NotificationChange',
              toEmail: req.user.email
            });
            return _context10.abrupt("return", res.status(204).send());

          case 17:
          case "end":
            return _context10.stop();
        }
      }
    }, _callee10);
  }));

  return function (_x13, _x14) {
    return _ref6.apply(this, arguments);
  };
}()));
router.post('/switch/:lbId', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref8 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee12(req, res) {
    var userId, chain, lbId, loadBalancer, appsInPool, newApps, newestApp, processedLb;
    return _regenerator["default"].wrap(function _callee12$(_context12) {
      while (1) {
        switch (_context12.prev = _context12.next) {
          case 0:
            userId = req.user._id;
            chain = req.body.chain;
            lbId = req.params.lbId;
            _context12.next = 5;
            return _LoadBalancer["default"].findById(lbId);

          case 5:
            loadBalancer = _context12.sent;
            _context12.next = 8;
            return _PreStakedApp["default"].find({
              chain: chain,
              status: _applicationStatuses.APPLICATION_STATUSES.READY
            });

          case 8:
            appsInPool = _context12.sent;

            if (loadBalancer) {
              _context12.next = 11;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NONEXISTENT_APPLICATION',
                message: 'Application not found'
              }]
            });

          case 11:
            if (!(loadBalancer.user.toString() !== userId.toString())) {
              _context12.next = 13;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'UNAUTHORIZED_ACCESS',
                message: 'Application does not belong to user'
              }]
            });

          case 13:
            if (!(loadBalancer.applicationIDs.length > appsInPool.length)) {
              _context12.next = 15;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'TOO_MANY_APPS',
                message: 'Too many applications in Load Balancer'
              }]
            });

          case 15:
            _context12.next = 17;
            return Promise.all(loadBalancer.applicationIDs.map( /*#__PURE__*/function () {
              var _switchApp = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee11(applicationId) {
                var replacementApplication, oldApplication, newReplacementApplication;
                return _regenerator["default"].wrap(function _callee11$(_context11) {
                  while (1) {
                    switch (_context11.prev = _context11.next) {
                      case 0:
                        replacementApplication = appsInPool.pop();

                        if (replacementApplication) {
                          _context11.next = 3;
                          break;
                        }

                        throw new Error('No application for the selected chain is available');

                      case 3:
                        _context11.next = 5;
                        return _Application["default"].findById(applicationId);

                      case 5:
                        oldApplication = _context11.sent;

                        if (oldApplication) {
                          _context11.next = 8;
                          break;
                        }

                        throw new Error('Cannot find application');

                      case 8:
                        if (!(oldApplication.user.toString() !== req.user._id.toString())) {
                          _context11.next = 10;
                          break;
                        }

                        throw _httpError["default"].FORBIDDEN({
                          errors: [{
                            id: 'FOREIGN_APPLICATION',
                            message: 'Application does not belong to user'
                          }]
                        });

                      case 10:
                        oldApplication.status = _applicationStatuses.APPLICATION_STATUSES.AWAITING_GRACE_PERIOD;
                        oldApplication.lastChangedStatusAt = Date.now();
                        _context11.next = 14;
                        return oldApplication.save();

                      case 14:
                        // Create a new Application for the user and copy the previous user config
                        newReplacementApplication = new _Application["default"]({
                          // As we're moving to a new chain, everything related to the account and gateway AAT
                          // information will change, so we use all the data from the application that we took
                          // from the pool.
                          chain: replacementApplication.chain,
                          freeTierApplicationAccount: replacementApplication.freeTierApplicationAccount,
                          gatewayAAT: replacementApplication.gatewayAAT,
                          status: _applicationStatuses.APPLICATION_STATUSES.READY,
                          lastChangedStatusAt: Date.now(),
                          freeTier: true,
                          // We wanna preserve user-related configuration fields, so we just copy them over
                          // from the old application.
                          name: oldApplication.name,
                          user: oldApplication.user,
                          gatewaySettings: oldApplication.gatewaySettings
                        });
                        _context11.next = 17;
                        return newReplacementApplication.save();

                      case 17:
                        return _context11.abrupt("return", {
                          appId: newReplacementApplication._id.toString(),
                          address: newReplacementApplication.freeTierApplicationAccount.address,
                          publicKey: newReplacementApplication.freeTierApplicationAccount.publicKey
                        });

                      case 18:
                      case "end":
                        return _context11.stop();
                    }
                  }
                }, _callee11);
              }));

              function switchApp(_x18) {
                return _switchApp.apply(this, arguments);
              }

              return switchApp;
            }()));

          case 17:
            newApps = _context12.sent;
            loadBalancer.applicationIDs = newApps.map(function (app) {
              return app.appId;
            });
            loadBalancer.updatedAt = new Date(Date.now());
            _context12.next = 22;
            return loadBalancer.save();

          case 22:
            _context12.next = 24;
            return _Application["default"].findById(loadBalancer.applicationIDs[0]);

          case 24:
            newestApp = _context12.sent;
            processedLb = {
              chain: newestApp.chain,
              name: loadBalancer.name,
              apps: newApps,
              createdAt: loadBalancer.createdAt,
              updatedAt: loadBalancer.updatedAt,
              freeTier: true,
              id: loadBalancer._id.toString(),
              gatewaySettings: newestApp.gatewaySettings,
              notificationSettings: newestApp.notificationSettings,
              status: newestApp.status
            };
            res.status(200).send(processedLb);

          case 27:
          case "end":
            return _context12.stop();
        }
      }
    }, _callee12);
  }));

  return function (_x16, _x17) {
    return _ref8.apply(this, arguments);
  };
}()));
router.get('/total-relays/:lbId', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref9 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee14(req, res) {
    var userId, lbId, loadBalancer, appIds, gqlClient, sevenDaysAgo, totalRelaysAndLatency, cumulativeRelaysAndLatency, processedRelaysAndLatency;
    return _regenerator["default"].wrap(function _callee14$(_context14) {
      while (1) {
        switch (_context14.prev = _context14.next) {
          case 0:
            userId = req.user._id;
            lbId = req.params.lbId;
            _context14.next = 4;
            return _LoadBalancer["default"].findById(lbId);

          case 4:
            loadBalancer = _context14.sent;

            if (loadBalancer) {
              _context14.next = 7;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NONEXISTENT_LOADBALANCER',
                message: 'User does not have an active Load Balancer'
              }]
            });

          case 7:
            if (!(loadBalancer.user.toString() !== userId.toString())) {
              _context14.next = 9;
              break;
            }

            throw _httpError["default"].FORBIDDEN({
              errors: [{
                id: 'UNAUTHORIZED_ACCESS',
                message: 'User does not have access to this load balancer'
              }]
            });

          case 9:
            appIds = loadBalancer.applicationIDs;
            gqlClient = (0, _types.getSdk)(new _graphqlRequest.GraphQLClient((0, _environment["default"])('HASURA_URL'), {
              // @ts-ignore
              headers: {
                'x-hasura-admin-secret': (0, _environment["default"])('HASURA_SECRET')
              }
            }));
            sevenDaysAgo = (0, _dateUtils.composeDaysFromNowUtcDate)(7);
            _context14.next = 14;
            return Promise.all(appIds.map( /*#__PURE__*/function () {
              var _getData = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee13(applicationId) {
                var _result$relay_apps_da, _result$relay_apps_da2;

                var application, result;
                return _regenerator["default"].wrap(function _callee13$(_context13) {
                  while (1) {
                    switch (_context13.prev = _context13.next) {
                      case 0:
                        _context13.next = 2;
                        return _Application["default"].findById(applicationId);

                      case 2:
                        application = _context13.sent;
                        _context13.next = 5;
                        return gqlClient.getTotalRelaysAndLatency({
                          _eq: application.freeTierApplicationAccount.publicKey,
                          _gte: sevenDaysAgo
                        });

                      case 5:
                        result = _context13.sent;
                        return _context13.abrupt("return", {
                          total_relays: (_result$relay_apps_da = result.relay_apps_daily_aggregate.aggregate.sum.total_relays) !== null && _result$relay_apps_da !== void 0 ? _result$relay_apps_da : 0,
                          elapsed_time: (_result$relay_apps_da2 = result.relay_apps_daily_aggregate.aggregate.avg.elapsed_time) !== null && _result$relay_apps_da2 !== void 0 ? _result$relay_apps_da2 : 0
                        });

                      case 7:
                      case "end":
                        return _context13.stop();
                    }
                  }
                }, _callee13);
              }));

              function getData(_x21) {
                return _getData.apply(this, arguments);
              }

              return getData;
            }()));

          case 14:
            totalRelaysAndLatency = _context14.sent;
            cumulativeRelaysAndLatency = totalRelaysAndLatency.reduce(function processResults(prev, cur) {
              return {
                total_relays: prev.total_relays + cur.total_relays,
                elapsed_time: prev.elapsed_time + cur.elapsed_time
              };
            });
            processedRelaysAndLatency = {
              total_relays: cumulativeRelaysAndLatency.total_relays,
              elapsed_time: cumulativeRelaysAndLatency.elapsed_time / appIds.length
            };
            res.status(200).send(processedRelaysAndLatency);

          case 18:
          case "end":
            return _context14.stop();
        }
      }
    }, _callee14);
  }));

  return function (_x19, _x20) {
    return _ref9.apply(this, arguments);
  };
}()));
router.get('/successful-relays/:lbId', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref10 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee16(req, res) {
    var userId, lbId, loadBalancer, appIds, gqlClient, sevenDaysAgo, totalRelaysAndLatency, cumulativeRelaysAndLatency, processedRelaysAndLatency;
    return _regenerator["default"].wrap(function _callee16$(_context16) {
      while (1) {
        switch (_context16.prev = _context16.next) {
          case 0:
            userId = req.user._id;
            lbId = req.params.lbId;
            _context16.next = 4;
            return _LoadBalancer["default"].findById(lbId);

          case 4:
            loadBalancer = _context16.sent;

            if (loadBalancer) {
              _context16.next = 7;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NONEXISTENT_LOADBALANCER',
                message: 'User does not have an active Load Balancer'
              }]
            });

          case 7:
            if (!(loadBalancer.user.toString() !== userId.toString())) {
              _context16.next = 9;
              break;
            }

            throw _httpError["default"].FORBIDDEN({
              errors: [{
                id: 'UNAUTHORIZED_ACCESS',
                message: 'User does not have access to this load balancer'
              }]
            });

          case 9:
            appIds = loadBalancer.applicationIDs;
            gqlClient = (0, _types.getSdk)(new _graphqlRequest.GraphQLClient((0, _environment["default"])('HASURA_URL'), {
              // @ts-ignore
              headers: {
                'x-hasura-admin-secret': (0, _environment["default"])('HASURA_SECRET')
              }
            }));
            sevenDaysAgo = (0, _dateUtils.composeDaysFromNowUtcDate)(7);
            _context16.next = 14;
            return Promise.all(appIds.map( /*#__PURE__*/function () {
              var _getData2 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee15(applicationId) {
                var _result$relay_apps_da3, _result$relay_apps_da4;

                var application, result;
                return _regenerator["default"].wrap(function _callee15$(_context15) {
                  while (1) {
                    switch (_context15.prev = _context15.next) {
                      case 0:
                        _context15.next = 2;
                        return _Application["default"].findById(applicationId);

                      case 2:
                        application = _context15.sent;
                        _context15.next = 5;
                        return gqlClient.getTotalSuccessfulRelays({
                          _eq: application.freeTierApplicationAccount.publicKey,
                          _gte: sevenDaysAgo
                        });

                      case 5:
                        result = _context15.sent;
                        return _context15.abrupt("return", {
                          total_relays: (_result$relay_apps_da3 = result.relay_apps_daily_aggregate.aggregate.sum.total_relays) !== null && _result$relay_apps_da3 !== void 0 ? _result$relay_apps_da3 : 0,
                          elapsed_time: (_result$relay_apps_da4 = result.relay_apps_daily_aggregate.aggregate.avg.elapsed_time) !== null && _result$relay_apps_da4 !== void 0 ? _result$relay_apps_da4 : 0
                        });

                      case 7:
                      case "end":
                        return _context15.stop();
                    }
                  }
                }, _callee15);
              }));

              function getData(_x24) {
                return _getData2.apply(this, arguments);
              }

              return getData;
            }()));

          case 14:
            totalRelaysAndLatency = _context16.sent;
            cumulativeRelaysAndLatency = totalRelaysAndLatency.reduce(function processResults(prev, cur) {
              return {
                total_relays: prev.total_relays + cur.total_relays,
                elapsed_time: prev.elapsed_time + cur.elapsed_time
              };
            }, {
              total_relays: 0,
              elapsed_time: 0
            });
            processedRelaysAndLatency = {
              total_relays: cumulativeRelaysAndLatency.total_relays,
              elapsed_time: cumulativeRelaysAndLatency.elapsed_time / appIds.length
            };
            res.status(200).send(processedRelaysAndLatency);

          case 18:
          case "end":
            return _context16.stop();
        }
      }
    }, _callee16);
  }));

  return function (_x22, _x23) {
    return _ref10.apply(this, arguments);
  };
}()));
router.get('/daily-relays/:lbId', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref11 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee18(req, res) {
    var userId, lbId, loadBalancer, appIds, gqlClient, sevenDaysAgo, dailyRelays, processedDailyRelays, _iterator3, _step3, _step3$value, bucket, dailyRelayCount;

    return _regenerator["default"].wrap(function _callee18$(_context18) {
      while (1) {
        switch (_context18.prev = _context18.next) {
          case 0:
            userId = req.user._id;
            lbId = req.params.lbId;
            _context18.next = 4;
            return _LoadBalancer["default"].findById(lbId);

          case 4:
            loadBalancer = _context18.sent;

            if (loadBalancer) {
              _context18.next = 7;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NONEXISTENT_LOADBALANCER',
                message: 'User does not have an active Load Balancer'
              }]
            });

          case 7:
            if (!(loadBalancer.user.toString() !== userId.toString())) {
              _context18.next = 9;
              break;
            }

            throw _httpError["default"].FORBIDDEN({
              errors: [{
                id: 'UNAUTHORIZED_ACCESS',
                message: 'User does not have access to this load balancer'
              }]
            });

          case 9:
            appIds = loadBalancer.applicationIDs;
            gqlClient = (0, _types.getSdk)(new _graphqlRequest.GraphQLClient((0, _environment["default"])('HASURA_URL'), {
              // @ts-ignore
              headers: {
                'x-hasura-admin-secret': (0, _environment["default"])('HASURA_SECRET')
              }
            }));
            sevenDaysAgo = (0, _dateUtils.composeDaysFromNowUtcDate)(7);
            dailyRelays = new Map();
            _context18.next = 15;
            return Promise.all(appIds.map( /*#__PURE__*/function () {
              var _getData3 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee17(applicationId) {
                var application, result, _iterator2, _step2, _step2$value, bucket, dailyRelayCount, currentCount;

                return _regenerator["default"].wrap(function _callee17$(_context17) {
                  while (1) {
                    switch (_context17.prev = _context17.next) {
                      case 0:
                        _context17.next = 2;
                        return _Application["default"].findById(applicationId);

                      case 2:
                        application = _context17.sent;
                        _context17.next = 5;
                        return gqlClient.getDailyTotalRelays({
                          _eq: application.freeTierApplicationAccount.publicKey,
                          _gte: sevenDaysAgo
                        });

                      case 5:
                        result = _context17.sent;
                        _iterator2 = _createForOfIteratorHelper(result.relay_apps_daily);

                        try {
                          for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
                            _step2$value = _step2.value, bucket = _step2$value.bucket, dailyRelayCount = _step2$value.total_relays;

                            if (!dailyRelays.has(bucket)) {
                              dailyRelays.set(bucket, dailyRelayCount !== null && dailyRelayCount !== void 0 ? dailyRelayCount : 0);
                            } else {
                              currentCount = dailyRelays.get(bucket);
                              dailyRelays.set(bucket, Number(currentCount) + Number(dailyRelayCount));
                            }
                          }
                        } catch (err) {
                          _iterator2.e(err);
                        } finally {
                          _iterator2.f();
                        }

                      case 8:
                      case "end":
                        return _context17.stop();
                    }
                  }
                }, _callee17);
              }));

              function getData(_x27) {
                return _getData3.apply(this, arguments);
              }

              return getData;
            }()));

          case 15:
            processedDailyRelays = [];
            _iterator3 = _createForOfIteratorHelper(dailyRelays.entries());

            try {
              for (_iterator3.s(); !(_step3 = _iterator3.n()).done;) {
                _step3$value = (0, _slicedToArray2["default"])(_step3.value, 2), bucket = _step3$value[0], dailyRelayCount = _step3$value[1];
                processedDailyRelays.push({
                  bucket: bucket,
                  dailyRelays: dailyRelayCount
                });
              }
            } catch (err) {
              _iterator3.e(err);
            } finally {
              _iterator3.f();
            }

            res.status(200).send({
              daily_relays: processedDailyRelays.reverse()
            });

          case 19:
          case "end":
            return _context18.stop();
        }
      }
    }, _callee18);
  }));

  return function (_x25, _x26) {
    return _ref11.apply(this, arguments);
  };
}()));
router.get('/session-relays/:lbId', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref12 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee20(req, res) {
    var userId, lbId, loadBalancer, appIds, gqlClient, today, sessionRelaysPerApp, totalSessionRelays;
    return _regenerator["default"].wrap(function _callee20$(_context20) {
      while (1) {
        switch (_context20.prev = _context20.next) {
          case 0:
            userId = req.user._id;
            lbId = req.params.lbId;
            _context20.next = 4;
            return _LoadBalancer["default"].findById(lbId);

          case 4:
            loadBalancer = _context20.sent;

            if (loadBalancer) {
              _context20.next = 7;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NONEXISTENT_LOADBALANCER',
                message: 'User does not have an active Load Balancer'
              }]
            });

          case 7:
            if (!(loadBalancer.user.toString() !== userId.toString())) {
              _context20.next = 9;
              break;
            }

            throw _httpError["default"].FORBIDDEN({
              errors: [{
                id: 'UNAUTHORIZED_ACCESS',
                message: 'User does not have access to this load balancer'
              }]
            });

          case 9:
            appIds = loadBalancer.applicationIDs;
            gqlClient = (0, _types.getSdk)(new _graphqlRequest.GraphQLClient((0, _environment["default"])('HASURA_URL'), {
              // @ts-ignore
              headers: {
                'x-hasura-admin-secret': (0, _environment["default"])('HASURA_SECRET')
              }
            }));
            today = (0, _dateUtils.composeTodayUtcDate)();
            _context20.next = 14;
            return Promise.all(appIds.map( /*#__PURE__*/function () {
              var _getData4 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee19(applicationId) {
                var application, result, totalSessionRelays;
                return _regenerator["default"].wrap(function _callee19$(_context19) {
                  while (1) {
                    switch (_context19.prev = _context19.next) {
                      case 0:
                        _context19.next = 2;
                        return _Application["default"].findById(applicationId);

                      case 2:
                        application = _context19.sent;
                        _context19.next = 5;
                        return gqlClient.getLastSessionAppRelays({
                          _eq: application.freeTierApplicationAccount.publicKey,
                          _gte: today,
                          _buckets: BUCKETS_PER_HOUR
                        });

                      case 5:
                        result = _context19.sent;
                        totalSessionRelays = result.relay_app_hourly.reduce(function (total, _ref13) {
                          var totalRelays = _ref13.total_relays;
                          return total + totalRelays;
                        }, 0);
                        return _context19.abrupt("return", totalSessionRelays);

                      case 8:
                      case "end":
                        return _context19.stop();
                    }
                  }
                }, _callee19);
              }));

              function getData(_x30) {
                return _getData4.apply(this, arguments);
              }

              return getData;
            }()));

          case 14:
            sessionRelaysPerApp = _context20.sent;
            totalSessionRelays = sessionRelaysPerApp.reduce(function sumRelays(total, sessionRelays) {
              return total + sessionRelays;
            }, 0);
            res.status(200).send({
              session_relays: totalSessionRelays
            });

          case 17:
          case "end":
            return _context20.stop();
        }
      }
    }, _callee20);
  }));

  return function (_x28, _x29) {
    return _ref12.apply(this, arguments);
  };
}()));
router.post('/latest-relays', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref14 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee22(req, res) {
    var userId, _req$body3, id, limit, offset, loadBalancer, appIds, gqlClient, latestRelaysPerApp, relays;

    return _regenerator["default"].wrap(function _callee22$(_context22) {
      while (1) {
        switch (_context22.prev = _context22.next) {
          case 0:
            userId = req.user._id;
            _req$body3 = req.body, id = _req$body3.id, limit = _req$body3.limit, offset = _req$body3.offset;
            _context22.next = 4;
            return _LoadBalancer["default"].findById(id);

          case 4:
            loadBalancer = _context22.sent;

            if (loadBalancer) {
              _context22.next = 7;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NONEXISTENT_LOADBALANCER',
                message: 'User does not have an active Load Balancer'
              }]
            });

          case 7:
            if (!(loadBalancer.user.toString() !== userId.toString())) {
              _context22.next = 9;
              break;
            }

            throw _httpError["default"].FORBIDDEN({
              errors: [{
                id: 'UNAUTHORIZED_ACCESS',
                message: 'User does not have access to this load balancer'
              }]
            });

          case 9:
            appIds = loadBalancer.applicationIDs;
            gqlClient = (0, _types.getSdk)(new _graphqlRequest.GraphQLClient((0, _environment["default"])('HASURA_URL'), {
              // @ts-ignore
              headers: {
                'x-hasura-admin-secret': (0, _environment["default"])('HASURA_SECRET')
              }
            }));
            _context22.next = 13;
            return Promise.all(appIds.map( /*#__PURE__*/function () {
              var _getData5 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee21(applicationId) {
                var application, result;
                return _regenerator["default"].wrap(function _callee21$(_context21) {
                  while (1) {
                    switch (_context21.prev = _context21.next) {
                      case 0:
                        _context21.next = 2;
                        return _Application["default"].findById(applicationId);

                      case 2:
                        application = _context21.sent;
                        _context21.next = 5;
                        return gqlClient.getLatestRelays({
                          _eq: application.freeTierApplicationAccount.publicKey,
                          limit: limit,
                          offset: offset
                        });

                      case 5:
                        result = _context21.sent;
                        return _context21.abrupt("return", result);

                      case 7:
                      case "end":
                        return _context21.stop();
                    }
                  }
                }, _callee21);
              }));

              function getData(_x33) {
                return _getData5.apply(this, arguments);
              }

              return getData;
            }()));

          case 13:
            latestRelaysPerApp = _context22.sent;
            relays = [];
            latestRelaysPerApp.map(function (relayBatch) {
              var _iterator4 = _createForOfIteratorHelper(relayBatch.relay),
                  _step4;

              try {
                for (_iterator4.s(); !(_step4 = _iterator4.n()).done;) {
                  var relay = _step4.value;
                  relays.push(relay);
                }
              } catch (err) {
                _iterator4.e(err);
              } finally {
                _iterator4.f();
              }
            });
            relays.sort(function (a, b) {
              var dateA = new Date(a.timestamp);
              var dateB = new Date(b.timestamp); // @ts-ignore

              return dateA - dateB;
            }).reverse();
            res.status(200).send({
              session_relays: relays.slice(0, 10)
            });

          case 18:
          case "end":
            return _context22.stop();
        }
      }
    }, _callee22);
  }));

  return function (_x31, _x32) {
    return _ref14.apply(this, arguments);
  };
}()));
router.post('/latest-successful-relays', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref15 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee24(req, res) {
    var userId, _req$body4, id, limit, offset, loadBalancer, appIds, gqlClient, latestRelaysPerApp, relays;

    return _regenerator["default"].wrap(function _callee24$(_context24) {
      while (1) {
        switch (_context24.prev = _context24.next) {
          case 0:
            userId = req.user._id;
            _req$body4 = req.body, id = _req$body4.id, limit = _req$body4.limit, offset = _req$body4.offset;
            _context24.next = 4;
            return _LoadBalancer["default"].findById(id);

          case 4:
            loadBalancer = _context24.sent;

            if (loadBalancer) {
              _context24.next = 7;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NONEXISTENT_LOADBALANCER',
                message: 'User does not have an active Load Balancer'
              }]
            });

          case 7:
            if (!(loadBalancer.user.toString() !== userId.toString())) {
              _context24.next = 9;
              break;
            }

            throw _httpError["default"].FORBIDDEN({
              errors: [{
                id: 'UNAUTHORIZED_ACCESS',
                message: 'User does not have access to this load balancer'
              }]
            });

          case 9:
            appIds = loadBalancer.applicationIDs;
            gqlClient = (0, _types.getSdk)(new _graphqlRequest.GraphQLClient((0, _environment["default"])('HASURA_URL'), {
              // @ts-ignore
              headers: {
                'x-hasura-admin-secret': (0, _environment["default"])('HASURA_SECRET')
              }
            }));
            _context24.next = 13;
            return Promise.all(appIds.map( /*#__PURE__*/function () {
              var _getData6 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee23(applicationId) {
                var application, result;
                return _regenerator["default"].wrap(function _callee23$(_context23) {
                  while (1) {
                    switch (_context23.prev = _context23.next) {
                      case 0:
                        _context23.next = 2;
                        return _Application["default"].findById(applicationId);

                      case 2:
                        application = _context23.sent;
                        _context23.next = 5;
                        return gqlClient.getLatestSuccessfulRelays({
                          _eq: application.freeTierApplicationAccount.publicKey,
                          _eq1: 200,
                          offset: offset
                        });

                      case 5:
                        result = _context23.sent;
                        return _context23.abrupt("return", result);

                      case 7:
                      case "end":
                        return _context23.stop();
                    }
                  }
                }, _callee23);
              }));

              function getData(_x36) {
                return _getData6.apply(this, arguments);
              }

              return getData;
            }()));

          case 13:
            latestRelaysPerApp = _context24.sent;
            relays = [];
            latestRelaysPerApp.map(function (relayBatch) {
              var _iterator5 = _createForOfIteratorHelper(relayBatch.relay),
                  _step5;

              try {
                for (_iterator5.s(); !(_step5 = _iterator5.n()).done;) {
                  var relay = _step5.value;
                  relays.push(relay);
                }
              } catch (err) {
                _iterator5.e(err);
              } finally {
                _iterator5.f();
              }
            });
            relays.sort(function (a, b) {
              var dateA = new Date(a.timestamp);
              var dateB = new Date(b.timestamp); // @ts-ignore

              return dateA - dateB;
            }).reverse();
            res.status(200).send({
              session_relays: relays.slice(0, 10)
            });

          case 18:
          case "end":
            return _context24.stop();
        }
      }
    }, _callee24);
  }));

  return function (_x34, _x35) {
    return _ref15.apply(this, arguments);
  };
}()));
router.post('/latest-failing-relays', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref16 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee26(req, res) {
    var userId, _req$body5, id, limit, offset, loadBalancer, appIds, gqlClient, latestRelaysPerApp, relays;

    return _regenerator["default"].wrap(function _callee26$(_context26) {
      while (1) {
        switch (_context26.prev = _context26.next) {
          case 0:
            userId = req.user._id;
            _req$body5 = req.body, id = _req$body5.id, limit = _req$body5.limit, offset = _req$body5.offset;
            _context26.next = 4;
            return _LoadBalancer["default"].findById(id);

          case 4:
            loadBalancer = _context26.sent;

            if (loadBalancer) {
              _context26.next = 7;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NONEXISTENT_LOADBALANCER',
                message: 'User does not have an active Load Balancer'
              }]
            });

          case 7:
            if (!(loadBalancer.user.toString() !== userId.toString())) {
              _context26.next = 9;
              break;
            }

            throw _httpError["default"].FORBIDDEN({
              errors: [{
                id: 'UNAUTHORIZED_ACCESS',
                message: 'User does not have access to this load balancer'
              }]
            });

          case 9:
            appIds = loadBalancer.applicationIDs;
            gqlClient = (0, _types.getSdk)(new _graphqlRequest.GraphQLClient((0, _environment["default"])('HASURA_URL'), {
              // @ts-ignore
              headers: {
                'x-hasura-admin-secret': (0, _environment["default"])('HASURA_SECRET')
              }
            }));
            _context26.next = 13;
            return Promise.all(appIds.map( /*#__PURE__*/function () {
              var _getData7 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee25(applicationId) {
                var application, result;
                return _regenerator["default"].wrap(function _callee25$(_context25) {
                  while (1) {
                    switch (_context25.prev = _context25.next) {
                      case 0:
                        _context25.next = 2;
                        return _Application["default"].findById(applicationId);

                      case 2:
                        application = _context25.sent;
                        _context25.next = 5;
                        return gqlClient.getLatestFailingRelays({
                          _eq: application.freeTierApplicationAccount.publicKey,
                          _eq1: 200,
                          offset: offset
                        });

                      case 5:
                        result = _context25.sent;
                        return _context25.abrupt("return", result);

                      case 7:
                      case "end":
                        return _context25.stop();
                    }
                  }
                }, _callee25);
              }));

              function getData(_x39) {
                return _getData7.apply(this, arguments);
              }

              return getData;
            }()));

          case 13:
            latestRelaysPerApp = _context26.sent;
            relays = [];
            latestRelaysPerApp.map(function (relayBatch) {
              var _iterator6 = _createForOfIteratorHelper(relayBatch.relay),
                  _step6;

              try {
                for (_iterator6.s(); !(_step6 = _iterator6.n()).done;) {
                  var relay = _step6.value;
                  relays.push(relay);
                }
              } catch (err) {
                _iterator6.e(err);
              } finally {
                _iterator6.f();
              }
            });
            relays.sort(function (a, b) {
              var dateA = new Date(a.timestamp);
              var dateB = new Date(b.timestamp); // @ts-ignore

              return dateA - dateB;
            }).reverse();
            res.status(200).send({
              session_relays: relays.slice(0, 10)
            });

          case 18:
          case "end":
            return _context26.stop();
        }
      }
    }, _callee26);
  }));

  return function (_x37, _x38) {
    return _ref16.apply(this, arguments);
  };
}()));
router.get('/ranged-relays/:lbId', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref17 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee28(req, res) {
    var userId, lbId, loadBalancer, appIds, gqlClient, fourteenDaysAgo, sevenDaysAgo, previousSuccessfulRelaysPerApp, totalRangedRelays;
    return _regenerator["default"].wrap(function _callee28$(_context28) {
      while (1) {
        switch (_context28.prev = _context28.next) {
          case 0:
            userId = req.user._id;
            lbId = req.params.lbId;
            _context28.next = 4;
            return _LoadBalancer["default"].findById(lbId);

          case 4:
            loadBalancer = _context28.sent;

            if (loadBalancer) {
              _context28.next = 7;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NONEXISTENT_LOADBALANCER',
                message: 'User does not have an active Load Balancer'
              }]
            });

          case 7:
            if (!(loadBalancer.user.toString() !== userId.toString())) {
              _context28.next = 9;
              break;
            }

            throw _httpError["default"].FORBIDDEN({
              errors: [{
                id: 'UNAUTHORIZED_ACCESS',
                message: 'User does not have access to this load balancer'
              }]
            });

          case 9:
            appIds = loadBalancer.applicationIDs;
            gqlClient = (0, _types.getSdk)(new _graphqlRequest.GraphQLClient((0, _environment["default"])('HASURA_URL'), {
              // @ts-ignore
              headers: {
                'x-hasura-admin-secret': (0, _environment["default"])('HASURA_SECRET')
              }
            }));
            fourteenDaysAgo = (0, _dateUtils.composeDaysFromNowUtcDate)(14);
            sevenDaysAgo = (0, _dateUtils.composeDaysFromNowUtcDate)(7);
            _context28.next = 15;
            return Promise.all(appIds.map( /*#__PURE__*/function () {
              var _getData8 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee27(applicationId) {
                var _result$relay_apps_da5;

                var application, result;
                return _regenerator["default"].wrap(function _callee27$(_context27) {
                  while (1) {
                    switch (_context27.prev = _context27.next) {
                      case 0:
                        _context27.next = 2;
                        return _Application["default"].findById(applicationId);

                      case 2:
                        application = _context27.sent;
                        _context27.next = 5;
                        return gqlClient.getTotalRangedRelaysAndLatency({
                          _eq: application.freeTierApplicationAccount.publicKey,
                          _gte: fourteenDaysAgo,
                          _lte: sevenDaysAgo
                        });

                      case 5:
                        result = _context27.sent;
                        return _context27.abrupt("return", {
                          total_relays: (_result$relay_apps_da5 = result.relay_apps_daily_aggregate.aggregate.sum.total_relays) !== null && _result$relay_apps_da5 !== void 0 ? _result$relay_apps_da5 : 0
                        });

                      case 7:
                      case "end":
                        return _context27.stop();
                    }
                  }
                }, _callee27);
              }));

              function getData(_x42) {
                return _getData8.apply(this, arguments);
              }

              return getData;
            }()));

          case 15:
            previousSuccessfulRelaysPerApp = _context28.sent;
            totalRangedRelays = previousSuccessfulRelaysPerApp.reduce(function sumRelays(total, app) {
              return total + app.total_relays;
            }, 0);
            res.status(200).send({
              total_relays: totalRangedRelays
            });

          case 18:
          case "end":
            return _context28.stop();
        }
      }
    }, _callee28);
  }));

  return function (_x40, _x41) {
    return _ref17.apply(this, arguments);
  };
}()));
router.get('/previous-successful-relays/:lbId', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref18 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee30(req, res) {
    var userId, lbId, loadBalancer, appIds, gqlClient, fourteenDaysAgo, sevenDaysAgo, previousSuccessfulRelaysPerApp, totalSuccessfulRelays;
    return _regenerator["default"].wrap(function _callee30$(_context30) {
      while (1) {
        switch (_context30.prev = _context30.next) {
          case 0:
            userId = req.user._id;
            lbId = req.params.lbId;
            _context30.next = 4;
            return _LoadBalancer["default"].findById(lbId);

          case 4:
            loadBalancer = _context30.sent;

            if (loadBalancer) {
              _context30.next = 7;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NONEXISTENT_LOADBALANCER',
                message: 'User does not have an active Load Balancer'
              }]
            });

          case 7:
            if (!(loadBalancer.user.toString() !== userId.toString())) {
              _context30.next = 9;
              break;
            }

            throw _httpError["default"].FORBIDDEN({
              errors: [{
                id: 'UNAUTHORIZED_ACCESS',
                message: 'User does not have access to this load balancer'
              }]
            });

          case 9:
            appIds = loadBalancer.applicationIDs;
            gqlClient = (0, _types.getSdk)(new _graphqlRequest.GraphQLClient((0, _environment["default"])('HASURA_URL'), {
              // @ts-ignore
              headers: {
                'x-hasura-admin-secret': (0, _environment["default"])('HASURA_SECRET')
              }
            }));
            fourteenDaysAgo = (0, _dateUtils.composeDaysFromNowUtcDate)(14);
            sevenDaysAgo = (0, _dateUtils.composeDaysFromNowUtcDate)(7);
            _context30.next = 15;
            return Promise.all(appIds.map( /*#__PURE__*/function () {
              var _getData9 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee29(applicationId) {
                var _result$relay_apps_da6;

                var application, result;
                return _regenerator["default"].wrap(function _callee29$(_context29) {
                  while (1) {
                    switch (_context29.prev = _context29.next) {
                      case 0:
                        _context29.next = 2;
                        return _Application["default"].findById(applicationId);

                      case 2:
                        application = _context29.sent;
                        _context29.next = 5;
                        return gqlClient.getTotalSuccessfulRangedRelays({
                          _eq: application.freeTierApplicationAccount.publicKey,
                          _gte: fourteenDaysAgo,
                          _lte: sevenDaysAgo
                        });

                      case 5:
                        result = _context29.sent;
                        return _context29.abrupt("return", {
                          successful_relays: (_result$relay_apps_da6 = result.relay_apps_daily_aggregate.aggregate.sum.total_relays) !== null && _result$relay_apps_da6 !== void 0 ? _result$relay_apps_da6 : 0
                        });

                      case 7:
                      case "end":
                        return _context29.stop();
                    }
                  }
                }, _callee29);
              }));

              function getData(_x45) {
                return _getData9.apply(this, arguments);
              }

              return getData;
            }()));

          case 15:
            previousSuccessfulRelaysPerApp = _context30.sent;
            totalSuccessfulRelays = previousSuccessfulRelaysPerApp.reduce(function sumRelays(total, app) {
              return total + app.successful_relays;
            }, 0);
            res.status(200).send({
              successful_relays: totalSuccessfulRelays
            });

          case 18:
          case "end":
            return _context30.stop();
        }
      }
    }, _callee30);
  }));

  return function (_x43, _x44) {
    return _ref18.apply(this, arguments);
  };
}()));
router.get('/hourly-latency/:lbId', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref19 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee32(req, res) {
    var userId, lbId, loadBalancer, appIds, gqlClient, aDayAgo, hourlyLatency, processedHourlyLatency, _iterator8, _step8, _step8$value, bucket, hourlyLatencyAvg;

    return _regenerator["default"].wrap(function _callee32$(_context32) {
      while (1) {
        switch (_context32.prev = _context32.next) {
          case 0:
            userId = req.user._id;
            lbId = req.params.lbId;
            _context32.next = 4;
            return _LoadBalancer["default"].findById(lbId);

          case 4:
            loadBalancer = _context32.sent;

            if (loadBalancer) {
              _context32.next = 7;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NONEXISTENT_LOADBALANCER',
                message: 'User does not have an active Load Balancer'
              }]
            });

          case 7:
            if (!(loadBalancer.user.toString() !== userId.toString())) {
              _context32.next = 9;
              break;
            }

            throw _httpError["default"].FORBIDDEN({
              errors: [{
                id: 'UNAUTHORIZED_ACCESS',
                message: 'User does not have access to this load balancer'
              }]
            });

          case 9:
            appIds = loadBalancer.applicationIDs;
            gqlClient = (0, _types.getSdk)(new _graphqlRequest.GraphQLClient((0, _environment["default"])('HASURA_URL'), {
              // @ts-ignore
              headers: {
                'x-hasura-admin-secret': (0, _environment["default"])('HASURA_SECRET')
              }
            }));
            aDayAgo = (0, _dateUtils.composeHoursFromNowUtcDate)(24);
            hourlyLatency = new Map();
            _context32.next = 15;
            return Promise.all(appIds.map( /*#__PURE__*/function () {
              var _getData10 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee31(applicationId) {
                var application, result, _iterator7, _step7, _step7$value, bucket, elapsedTime, currentCount;

                return _regenerator["default"].wrap(function _callee31$(_context31) {
                  while (1) {
                    switch (_context31.prev = _context31.next) {
                      case 0:
                        _context31.next = 2;
                        return _Application["default"].findById(applicationId);

                      case 2:
                        application = _context31.sent;
                        _context31.next = 5;
                        return gqlClient.getTotalRelayDuration({
                          _eq: application.freeTierApplicationAccount.publicKey,
                          _gte: aDayAgo
                        });

                      case 5:
                        result = _context31.sent;
                        _iterator7 = _createForOfIteratorHelper(result.relay_app_hourly);

                        try {
                          for (_iterator7.s(); !(_step7 = _iterator7.n()).done;) {
                            _step7$value = _step7.value, bucket = _step7$value.bucket, elapsedTime = _step7$value.elapsed_time;

                            if (!hourlyLatency.has(bucket)) {
                              hourlyLatency.set(bucket, elapsedTime !== null && elapsedTime !== void 0 ? elapsedTime : 0);
                            } else {
                              currentCount = hourlyLatency.get(bucket);
                              hourlyLatency.set(bucket, (Number(currentCount) + Number(elapsedTime)) / 2);
                            }
                          }
                        } catch (err) {
                          _iterator7.e(err);
                        } finally {
                          _iterator7.f();
                        }

                      case 8:
                      case "end":
                        return _context31.stop();
                    }
                  }
                }, _callee31);
              }));

              function getData(_x48) {
                return _getData10.apply(this, arguments);
              }

              return getData;
            }()));

          case 15:
            processedHourlyLatency = [];
            _iterator8 = _createForOfIteratorHelper(hourlyLatency.entries());

            try {
              for (_iterator8.s(); !(_step8 = _iterator8.n()).done;) {
                _step8$value = (0, _slicedToArray2["default"])(_step8.value, 2), bucket = _step8$value[0], hourlyLatencyAvg = _step8$value[1];
                processedHourlyLatency.push({
                  bucket: bucket,
                  latency: hourlyLatencyAvg
                });
              }
            } catch (err) {
              _iterator8.e(err);
            } finally {
              _iterator8.f();
            }

            res.status(200).send({
              hourly_latency: processedHourlyLatency.reverse()
            });

          case 19:
          case "end":
            return _context32.stop();
        }
      }
    }, _callee32);
  }));

  return function (_x46, _x47) {
    return _ref19.apply(this, arguments);
  };
}()));
var _default = router;
exports["default"] = _default;
//# sourceMappingURL=LoadBalancerController.js.map