"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _express = _interopRequireDefault(require("express"));

var _graphqlRequest = require("graphql-request");

var _environment = _interopRequireDefault(require("../environment"));

var _types = require("../graphql/types");

var _Blockchains = _interopRequireDefault(require("../models/Blockchains"));

var _NetworkData = _interopRequireDefault(require("../models/NetworkData"));

var _PreStakedApp = _interopRequireDefault(require("../models/PreStakedApp"));

var _async = _interopRequireDefault(require("../middlewares/async"));

var _passportAuth = require("../middlewares/passport-auth");

var _dateUtils = require("../lib/date-utils");

var router = _express["default"].Router();

router.use(_passportAuth.authenticate);
/**
 * Get info for all chains.
 */

router.get('/chains', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2(_, res) {
    var chains, processedChains;
    return _regenerator["default"].wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return _Blockchains["default"].find({
              nodeCount: {
                $exists: true
              }
            });

          case 2:
            chains = _context2.sent;
            _context2.next = 5;
            return Promise.all(chains.map( /*#__PURE__*/function () {
              var _processChain = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(_ref2) {
                var _id, ticker, network, description, nodeCount, isAvailableForStaking;

                return _regenerator["default"].wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        _id = _ref2._id, ticker = _ref2.ticker, network = _ref2.network, description = _ref2.description, nodeCount = _ref2.nodeCount;
                        _context.next = 3;
                        return _PreStakedApp["default"].exists({
                          chain: _id
                        });

                      case 3:
                        isAvailableForStaking = _context.sent;
                        return _context.abrupt("return", {
                          id: _id,
                          ticker: ticker,
                          network: network,
                          description: description,
                          nodeCount: nodeCount,
                          isAvailableForStaking: isAvailableForStaking
                        });

                      case 5:
                      case "end":
                        return _context.stop();
                    }
                  }
                }, _callee);
              }));

              function processChain(_x3) {
                return _processChain.apply(this, arguments);
              }

              return processChain;
            }()));

          case 5:
            processedChains = _context2.sent;
            res.status(200).send({
              chains: processedChains
            });

          case 7:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function (_x, _x2) {
    return _ref.apply(this, arguments);
  };
}()));
router.get('/stakeable-chains', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref3 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee4(_, res) {
    var chains, existentChains, processedChains, formattedChains;
    return _regenerator["default"].wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            _context4.next = 2;
            return _Blockchains["default"].find();

          case 2:
            chains = _context4.sent;
            _context4.next = 5;
            return Promise.all(chains.map( /*#__PURE__*/function () {
              var _filterChain = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee3(_ref4) {
                var _id, exists;

                return _regenerator["default"].wrap(function _callee3$(_context3) {
                  while (1) {
                    switch (_context3.prev = _context3.next) {
                      case 0:
                        _id = _ref4._id;
                        _context3.next = 3;
                        return _PreStakedApp["default"].exists({
                          chain: _id
                        });

                      case 3:
                        exists = _context3.sent;
                        return _context3.abrupt("return", exists);

                      case 5:
                      case "end":
                        return _context3.stop();
                    }
                  }
                }, _callee3);
              }));

              function filterChain(_x6) {
                return _filterChain.apply(this, arguments);
              }

              return filterChain;
            }()));

          case 5:
            existentChains = _context4.sent;
            processedChains = chains.filter(function (_, i) {
              return existentChains[i];
            });
            formattedChains = processedChains.map(function processChain(_ref5) {
              var _id = _ref5._id,
                  ticker = _ref5.ticker,
                  network = _ref5.network,
                  description = _ref5.description,
                  nodeCount = _ref5.nodeCount;
              return {
                id: _id,
                ticker: ticker,
                network: network,
                description: description,
                nodeCount: nodeCount,
                isAvailableForStaking: true
              };
            });
            res.status(200).send({
              chains: formattedChains
            });

          case 9:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4);
  }));

  return function (_x4, _x5) {
    return _ref3.apply(this, arguments);
  };
}()));
router.get('/summary', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref6 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee5(_, res) {
    var latestNetworkData;
    return _regenerator["default"].wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            _context5.next = 2;
            return _NetworkData["default"].findOne({}, {}, {
              sort: {
                createdAt: -1
              }
            });

          case 2:
            latestNetworkData = _context5.sent;
            res.status(200).send({
              summary: {
                appsStaked: latestNetworkData.appsStaked,
                nodesStaked: latestNetworkData.nodesStaked,
                poktStaked: latestNetworkData.poktStaked
              }
            });

          case 4:
          case "end":
            return _context5.stop();
        }
      }
    }, _callee5);
  }));

  return function (_x7, _x8) {
    return _ref6.apply(this, arguments);
  };
}()));
router.get('/daily-relays', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref7 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee6(_, res) {
    var gqlClient, networkRelays;
    return _regenerator["default"].wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            gqlClient = (0, _types.getSdk)(new _graphqlRequest.GraphQLClient((0, _environment["default"])('HASURA_URL'), {
              // @ts-ignore
              headers: {
                'x-hasura-admin-secret': (0, _environment["default"])('HASURA_SECRET')
              }
            }));
            _context6.next = 3;
            return gqlClient.getDailyNetworkRelays();

          case 3:
            networkRelays = _context6.sent;
            res.status(200).send(networkRelays.relays_daily);

          case 5:
          case "end":
            return _context6.stop();
        }
      }
    }, _callee6);
  }));

  return function (_x9, _x10) {
    return _ref7.apply(this, arguments);
  };
}()));
router.get('/weekly-successful-relays', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref8 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee7(_, res) {
    var gqlClient, sevenDaysAgo, networkRelays;
    return _regenerator["default"].wrap(function _callee7$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:
            gqlClient = (0, _types.getSdk)(new _graphqlRequest.GraphQLClient((0, _environment["default"])('HASURA_URL'), {
              // @ts-ignore
              headers: {
                'x-hasura-admin-secret': (0, _environment["default"])('HASURA_SECRET')
              }
            })); // @ts-ignore

            dayjs.extend(dayJsutcPlugin);
            sevenDaysAgo = (0, _dateUtils.composeDaysFromNowUtcDate)(7);
            _context7.next = 5;
            return gqlClient.getTotalSuccesfulNetworkRelays({
              _gte: sevenDaysAgo
            });

          case 5:
            networkRelays = _context7.sent;
            res.status(200).send(networkRelays.relay_apps_hourly_aggregate.aggregate.sum);

          case 7:
          case "end":
            return _context7.stop();
        }
      }
    }, _callee7);
  }));

  return function (_x11, _x12) {
    return _ref8.apply(this, arguments);
  };
}()));
router.get('/total-weekly-relays', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref9 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee8(_, res) {
    var gqlClient, sevenDaysAgo, networkRelays;
    return _regenerator["default"].wrap(function _callee8$(_context8) {
      while (1) {
        switch (_context8.prev = _context8.next) {
          case 0:
            gqlClient = (0, _types.getSdk)(new _graphqlRequest.GraphQLClient((0, _environment["default"])('HASURA_URL'), {
              // @ts-ignore
              headers: {
                'x-hasura-admin-secret': (0, _environment["default"])('HASURA_SECRET')
              }
            }));
            sevenDaysAgo = (0, _dateUtils.composeDaysFromNowUtcDate)(7);
            _context8.next = 4;
            return gqlClient.getTotalNetworkRelays({
              _gte: sevenDaysAgo
            });

          case 4:
            networkRelays = _context8.sent;
            res.status(200).send(networkRelays.relay_apps_hourly_aggregate.aggregate.sum);

          case 6:
          case "end":
            return _context8.stop();
        }
      }
    }, _callee8);
  }));

  return function (_x13, _x14) {
    return _ref9.apply(this, arguments);
  };
}()));
var _default = router;
exports["default"] = _default;
//# sourceMappingURL=NetworkController.js.map