"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _express = _interopRequireDefault(require("express"));

var _bcrypt = _interopRequireDefault(require("bcrypt"));

var _crypto = _interopRequireDefault(require("crypto"));

var _async = _interopRequireDefault(require("../middlewares/async"));

var _passportAuth = require("../middlewares/passport-auth");

var _Token = _interopRequireWildcard(require("../models/Token"));

var _User = _interopRequireDefault(require("../models/User"));

var _httpError = _interopRequireDefault(require("../errors/http-error"));

var _passportLocal = _interopRequireDefault(require("../lib/passport-local"));

var _MailgunService = _interopRequireDefault(require("../services/MailgunService"));

var _environment = _interopRequireDefault(require("../environment"));

var SALT_ROUNDS = 10;
var TEN_DAYS = 10 * 24 * 60 * 60 * 1000;

var router = _express["default"].Router();

function createCookieFromToken(user, statusCode, req, res) {
  var token = user.generateVerificationToken();
  var cookieOptions = {
    // Expires in 10 days
    expires: new Date(Date.now() + TEN_DAYS),
    httpOnly: true,
    sameSite: 'none',
    secure: req.secure || req.headers['x-forwarded-proto'] === 'https'
  };
  res.cookie('jwt', token, cookieOptions);
  res.status(statusCode).json({
    status: 'success',
    token: token,
    data: {
      user: user
    }
  });
}

function destroyCookie(user, req, res) {
  var token = user.generateVerificationToken();
  var cookieOptions = {
    // Expires in 10 days
    expires: new Date(Date.now() - TEN_DAYS),
    httpOnly: true,
    secure: req.secure || req.headers['x-forwarded-proto'] === 'https'
  };
  res.cookie('jwt', token, cookieOptions);
  res.status(200).json({
    status: 'success',
    token: token,
    data: {
      user: user
    }
  });
}

function createNewVerificationToken(_x, _x2) {
  return _createNewVerificationToken.apply(this, arguments);
}
/**
 * User authentication using username and password.
 */


function _createNewVerificationToken() {
  _createNewVerificationToken = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee9(userId, userEmail) {
    var staleToken, validationToken, hashedValidationToken, userValidationToken;
    return _regenerator["default"].wrap(function _callee9$(_context9) {
      while (1) {
        switch (_context9.prev = _context9.next) {
          case 0:
            _context9.next = 2;
            return _Token["default"].findOne({
              userId: userId
            });

          case 2:
            staleToken = _context9.sent;

            if (!staleToken) {
              _context9.next = 6;
              break;
            }

            _context9.next = 6;
            return staleToken.deleteOne();

          case 6:
            validationToken = _crypto["default"].randomBytes(32).toString('hex');
            _context9.next = 9;
            return _bcrypt["default"].hash(validationToken, SALT_ROUNDS);

          case 9:
            hashedValidationToken = _context9.sent;
            userValidationToken = new _Token["default"]({
              userId: userId,
              email: userEmail,
              token: hashedValidationToken,
              type: _Token.TOKEN_TYPES.verification,
              createdAt: Date.now()
            });
            _context9.next = 13;
            return userValidationToken.save();

          case 13:
            return _context9.abrupt("return", validationToken);

          case 14:
          case "end":
            return _context9.stop();
        }
      }
    }, _callee9);
  }));
  return _createNewVerificationToken.apply(this, arguments);
}

router.post('/login', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2(req, res, next) {
    return _regenerator["default"].wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _passportLocal["default"].authenticate('login', {
              session: false
            }, /*#__PURE__*/function () {
              var _ref2 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(err, user) {
                var validationToken, validationLink, emailService;
                return _regenerator["default"].wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        if (err) {
                          console.log('aqui miop?');
                          next(_httpError["default"].BAD_REQUEST({
                            errors: [{
                              id: 'INVALID_CREDENTIALS',
                              message: 'No account for this user found.'
                            }]
                          }));
                        }

                        if (!user) {
                          next(_httpError["default"].BAD_REQUEST({
                            errors: [{
                              id: 'INVALID_CREDENTIALS',
                              message: 'Wrong email or password'
                            }]
                          }));
                        }

                        if (user.validated) {
                          _context.next = 11;
                          break;
                        }

                        _context.next = 5;
                        return createNewVerificationToken(user._id, user.email);

                      case 5:
                        validationToken = _context.sent;
                        validationLink = "".concat((0, _environment["default"])('FRONTEND_URL'), "/#/validate?token=").concat(validationToken, "&email=").concat(encodeURIComponent(user.email));
                        emailService = new _MailgunService["default"]();
                        _context.next = 10;
                        return emailService.send({
                          templateData: {
                            user_email: user.email,
                            verify_link: validationLink
                          },
                          templateName: 'SignUp',
                          toEmail: user.email
                        });

                      case 10:
                        throw _httpError["default"].BAD_REQUEST({
                          errors: [{
                            id: 'NOT_VALIDATED',
                            message: 'Please verify your email'
                          }]
                        });

                      case 11:
                        createCookieFromToken(user, 200, req, res);

                      case 12:
                      case "end":
                        return _context.stop();
                    }
                  }
                }, _callee);
              }));

              return function (_x6, _x7) {
                return _ref2.apply(this, arguments);
              };
            }())(req, res, next);

          case 1:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function (_x3, _x4, _x5) {
    return _ref.apply(this, arguments);
  };
}()));
/**
 * User sign up using email.
 */

router.post('/signup', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref3 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee4(req, res, next) {
    return _regenerator["default"].wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            _passportLocal["default"].authenticate('signup', {
              session: false
            }, /*#__PURE__*/function () {
              var _ref4 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee3(err, user) {
                var validationToken, validationLink, emailService;
                return _regenerator["default"].wrap(function _callee3$(_context3) {
                  while (1) {
                    switch (_context3.prev = _context3.next) {
                      case 0:
                        if (err) {
                          next(err);
                        }

                        if (!user) {
                          next(_httpError["default"].INTERNAL_SERVER_ERROR({
                            errors: [{
                              id: 'CREATION_ERROR',
                              message: 'There was an error while creating your account'
                            }]
                          }));
                        }

                        _context3.next = 4;
                        return createNewVerificationToken(user._id, user.email);

                      case 4:
                        validationToken = _context3.sent;
                        validationLink = "".concat((0, _environment["default"])('FRONTEND_URL'), "/#/validate?token=").concat(validationToken, "&email=").concat(encodeURIComponent(user.email));
                        emailService = new _MailgunService["default"]();
                        _context3.next = 9;
                        return emailService.send({
                          templateData: {
                            user_email: user.email,
                            verify_link: validationLink
                          },
                          templateName: 'SignUp',
                          toEmail: user.email
                        });

                      case 9:
                        return _context3.abrupt("return", res.status(204).send());

                      case 10:
                      case "end":
                        return _context3.stop();
                    }
                  }
                }, _callee3);
              }));

              return function (_x11, _x12) {
                return _ref4.apply(this, arguments);
              };
            }())(req, res, next);

          case 1:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4);
  }));

  return function (_x8, _x9, _x10) {
    return _ref3.apply(this, arguments);
  };
}()));
router.post('/send-reset-email', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref5 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee5(req, res) {
    var email, processedEmail, user, resetToken, hashedResetToken, userResetToken, resetLink, emailService;
    return _regenerator["default"].wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            email = req.body.email;
            processedEmail = email;
            _context5.next = 4;
            return _User["default"].findOne({
              email: processedEmail
            });

          case 4:
            user = _context5.sent;

            if (user) {
              _context5.next = 7;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'EMAIL_DOES_NOT_EXIST',
                message: 'Email does not exist'
              }]
            });

          case 7:
            resetToken = _crypto["default"].randomBytes(32).toString('hex');
            _context5.next = 10;
            return _bcrypt["default"].hash(resetToken, SALT_ROUNDS);

          case 10:
            hashedResetToken = _context5.sent;
            userResetToken = new _Token["default"]({
              userId: user._id,
              email: processedEmail,
              token: hashedResetToken,
              type: _Token.TOKEN_TYPES.reset,
              createdAt: Date.now()
            });
            _context5.next = 14;
            return userResetToken.save();

          case 14:
            resetLink = "".concat((0, _environment["default"])('FRONTEND_URL'), "/#/newpassword?token=").concat(resetToken, "&email=").concat(encodeURIComponent(user.email));
            emailService = new _MailgunService["default"]();
            _context5.prev = 16;
            _context5.next = 19;
            return emailService.send({
              templateData: {
                user_email: user.email,
                reset_link: resetLink
              },
              templateName: 'PasswordReset',
              toEmail: user.email
            });

          case 19:
            _context5.next = 24;
            break;

          case 21:
            _context5.prev = 21;
            _context5.t0 = _context5["catch"](16);
            console.log(_context5.t0);

          case 24:
            return _context5.abrupt("return", res.status(204).send());

          case 25:
          case "end":
            return _context5.stop();
        }
      }
    }, _callee5, null, [[16, 21]]);
  }));

  return function (_x13, _x14) {
    return _ref5.apply(this, arguments);
  };
}()));
router.post('/reset-password', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref6 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee6(req, res) {
    var _req$body, plainToken, password1, password2, email, isPasswordValid, processedEmail, storedToken, isTokenMatching, newHashedPassword;

    return _regenerator["default"].wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            _req$body = req.body, plainToken = _req$body.plainToken, password1 = _req$body.password1, password2 = _req$body.password2, email = _req$body.email;

            if (!(!plainToken || !password1 || !password2 || !email)) {
              _context6.next = 3;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'MISSING_FIELDS',
                message: 'Missing required fields in body'
              }]
            });

          case 3:
            _context6.next = 5;
            return _User["default"].validatePassword(password1);

          case 5:
            isPasswordValid = _context6.sent;
            processedEmail = email;

            if (isPasswordValid) {
              _context6.next = 9;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NOT_SECURE_PASSWORD',
                message: 'Password is not secure enough'
              }]
            });

          case 9:
            if (!(password1 !== password2)) {
              _context6.next = 11;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NON_MATCHING_PASSWORDS',
                message: "Passwords don't match"
              }]
            });

          case 11:
            _context6.next = 13;
            return _Token["default"].findOne({
              $and: [{
                email: processedEmail
              }, {
                type: 'TOKEN_RESET'
              }]
            });

          case 13:
            storedToken = _context6.sent;

            if (storedToken) {
              _context6.next = 16;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'EXPIRED_TOKEN',
                message: 'Token has expired'
              }]
            });

          case 16:
            _context6.next = 18;
            return _bcrypt["default"].compare(plainToken, storedToken.token);

          case 18:
            isTokenMatching = _context6.sent;

            if (isTokenMatching) {
              _context6.next = 21;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'NON_MATCHING_TOKEN',
                message: 'Token is not matching'
              }]
            });

          case 21:
            _context6.next = 23;
            return _bcrypt["default"].hash(password1, SALT_ROUNDS);

          case 23:
            newHashedPassword = _context6.sent;
            _context6.next = 26;
            return _User["default"].updateOne({
              email: processedEmail
            }, {
              $set: {
                password: newHashedPassword
              }
            }, {
              "new": true
            });

          case 26:
            _context6.next = 28;
            return storedToken.deleteOne();

          case 28:
            res.status(204).send();

          case 29:
          case "end":
            return _context6.stop();
        }
      }
    }, _callee6);
  }));

  return function (_x15, _x16) {
    return _ref6.apply(this, arguments);
  };
}()));
router.post('/validate-user', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref7 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee7(req, res) {
    var _req$body2, plainToken, email, processedEmail, storedToken, isTokenMatching;

    return _regenerator["default"].wrap(function _callee7$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:
            _req$body2 = req.body, plainToken = _req$body2.plainToken, email = _req$body2.email;

            if (!(!plainToken || !email)) {
              _context7.next = 3;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'MISSING_FIELDS',
                message: 'Invalid request'
              }]
            });

          case 3:
            processedEmail = decodeURIComponent(email);
            _context7.next = 6;
            return _Token["default"].findOne({
              $and: [{
                email: processedEmail
              }, {
                type: 'TOKEN_VERIFICATION'
              }]
            });

          case 6:
            storedToken = _context7.sent;

            if (storedToken) {
              _context7.next = 9;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'EXPIRED_TOKEN',
                message: 'Token has expired'
              }]
            });

          case 9:
            _context7.next = 11;
            return _bcrypt["default"].compare(plainToken, storedToken.token);

          case 11:
            isTokenMatching = _context7.sent;

            if (isTokenMatching) {
              _context7.next = 14;
              break;
            }

            throw _httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'INVALID_TOKEN',
                message: 'Token is invalid'
              }]
            });

          case 14:
            _context7.next = 16;
            return _User["default"].updateOne({
              email: processedEmail
            }, {
              $set: {
                validated: true
              }
            }, {
              "new": true
            });

          case 16:
            _context7.next = 18;
            return storedToken.deleteOne();

          case 18:
            res.status(204).send();

          case 19:
          case "end":
            return _context7.stop();
        }
      }
    }, _callee7);
  }));

  return function (_x17, _x18) {
    return _ref7.apply(this, arguments);
  };
}()));
router.use(_passportAuth.authenticate);
router.post('/logout', (0, _async["default"])( /*#__PURE__*/function () {
  var _ref8 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee8(req, res) {
    var user;
    return _regenerator["default"].wrap(function _callee8$(_context8) {
      while (1) {
        switch (_context8.prev = _context8.next) {
          case 0:
            _context8.next = 2;
            return _User["default"].findById(req.user._id);

          case 2:
            user = _context8.sent;
            destroyCookie(user, req, res);

          case 4:
          case "end":
            return _context8.stop();
        }
      }
    }, _callee8);
  }));

  return function (_x19, _x20) {
    return _ref8.apply(this, arguments);
  };
}()));
var _default = router;
exports["default"] = _default;
//# sourceMappingURL=UserController.js.map