"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _passport = _interopRequireDefault(require("passport"));

var _passportLocal = require("passport-local");

var _passportJwt = require("passport-jwt");

var _User = _interopRequireDefault(require("../models/User"));

var _environment = _interopRequireDefault(require("../environment"));

var _httpError = _interopRequireDefault(require("../errors/http-error"));

function extractTokenFromCookie(req) {
  var _req$cookies$jwt, _req$cookies;

  return (_req$cookies$jwt = req === null || req === void 0 ? void 0 : (_req$cookies = req.cookies) === null || _req$cookies === void 0 ? void 0 : _req$cookies.jwt) !== null && _req$cookies$jwt !== void 0 ? _req$cookies$jwt : null;
}

var AUTH_FIELDS = {
  usernameField: 'email',
  passwordField: 'password',
  passReqToCallback: true
};
var JWT_OPTIONS = {
  secretOrKey: (0, _environment["default"])('AUTH').publicSecret,
  algorithms: ['RS256'],
  passReqToCallback: true,
  jwtFromRequest: _passportJwt.ExtractJwt.fromExtractors([_passportJwt.ExtractJwt.fromAuthHeaderAsBearerToken(), function (req) {
    return extractTokenFromCookie(req);
  }])
};

_passport["default"].use('login', new _passportLocal.Strategy(AUTH_FIELDS, /*#__PURE__*/function () {
  var _ref = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(_, email, password, done) {
    var user, isPasswordValid;
    return _regenerator["default"].wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.prev = 0;
            _context.next = 3;
            return _User["default"].findOne({
              email: email
            });

          case 3:
            user = _context.sent;

            if (user) {
              _context.next = 6;
              break;
            }

            return _context.abrupt("return", done(_httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'WRONG_EMAIL_OR_PASSWORD',
                message: 'Incorrect email or password'
              }]
            }), null));

          case 6:
            if (user.v2) {
              _context.next = 8;
              break;
            }

            return _context.abrupt("return", done(_httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'OUTDATED_USER',
                message: 'Your user has not been migrated to use the new dashboard.'
              }]
            }), null));

          case 8:
            _context.next = 10;
            return _User["default"].comparePassword(password, user.password);

          case 10:
            isPasswordValid = _context.sent;

            if (isPasswordValid) {
              _context.next = 13;
              break;
            }

            return _context.abrupt("return", done(_httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'WRONG_EMAIL_OR_PASSWORD',
                message: 'Incorrect email or password'
              }]
            }), null));

          case 13:
            return _context.abrupt("return", done(null, user));

          case 16:
            _context.prev = 16;
            _context.t0 = _context["catch"](0);
            return _context.abrupt("return", done(_httpError["default"].INTERNAL_SERVER_ERROR({
              errors: [{
                id: 'REQUEST_ERR',
                message: 'There was an error with your request'
              }]
            })));

          case 19:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, null, [[0, 16]]);
  }));

  return function (_x, _x2, _x3, _x4) {
    return _ref.apply(this, arguments);
  };
}()));

_passport["default"].use('signup', new _passportLocal.Strategy(AUTH_FIELDS, /*#__PURE__*/function () {
  var _ref2 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2(_, email, password, done) {
    var processedEmail, emailRegistered, isPasswordSecure, encryptedPassword, user;
    return _regenerator["default"].wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.prev = 0;
            processedEmail = email;
            _context2.next = 4;
            return _User["default"].findOne({
              email: processedEmail
            });

          case 4:
            emailRegistered = _context2.sent;

            if (!emailRegistered) {
              _context2.next = 7;
              break;
            }

            return _context2.abrupt("return", done(_httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'BAD_EMAIL',
                message: 'Email already in use'
              }]
            }), null));

          case 7:
            _context2.next = 9;
            return _User["default"].validatePassword(password);

          case 9:
            isPasswordSecure = _context2.sent;

            if (isPasswordSecure) {
              _context2.next = 12;
              break;
            }

            return _context2.abrupt("return", done(_httpError["default"].BAD_REQUEST({
              errors: [{
                id: 'BAD_PASSWORD',
                message: "Password's not strong enough"
              }]
            }), null));

          case 12:
            _context2.next = 14;
            return _User["default"].encryptPassword(password);

          case 14:
            encryptedPassword = _context2.sent;
            user = new _User["default"]({
              email: processedEmail,
              username: processedEmail,
              password: encryptedPassword,
              validated: false,
              v2: true
            });
            _context2.next = 18;
            return user.save();

          case 18:
            return _context2.abrupt("return", done(null, user));

          case 21:
            _context2.prev = 21;
            _context2.t0 = _context2["catch"](0);
            console.log(_context2.t0);
            return _context2.abrupt("return", done({
              statusCode: 400,
              message: _context2.t0.message
            }));

          case 25:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2, null, [[0, 21]]);
  }));

  return function (_x5, _x6, _x7, _x8) {
    return _ref2.apply(this, arguments);
  };
}()));

_passport["default"].use(new _passportJwt.Strategy(JWT_OPTIONS, function (_, jwtPayload, done) {
  _User["default"].findOne({
    _id: jwtPayload.id
  }).then(function (user) {
    if (!user) {
      done(null, false);
    } else {
      done(null, user);
    }
  })["catch"](function (err) {
    return done(err, false);
  });
}));

var _default = _passport["default"];
exports["default"] = _default;
//# sourceMappingURL=passport-local.js.map