"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getNodes = getNodes;
exports.getApplications = getApplications;
exports.transferFromFreeTierFund = transferFromFreeTierFund;
exports.createUnlockedAccount = createUnlockedAccount;
exports.getBalance = getBalance;
exports.getTX = getTX;
exports.getAccount = getAccount;
exports.getApp = getApp;
exports.createAppStakeTx = createAppStakeTx;
exports.getPocketInstance = getPocketInstance;
exports.submitRawTransaction = submitRawTransaction;
exports.POKT_DENOMINATIONS = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _pocketJs = require("@pokt-network/pocket-js");

var _environment = _interopRequireDefault(require("../environment"));

/* global BigInt */
var _ref = (0, _environment["default"])('POCKET_NETWORK'),
    blockTime = _ref.blockTime,
    chainId = _ref.chainId,
    dispatchers = _ref.dispatchers,
    freeTierFundAccount = _ref.freeTierFundAccount,
    freeTierFundAddress = _ref.freeTierFundAddress,
    httpProviderNode = _ref.httpProviderNode,
    maxDispatchers = _ref.maxDispatchers,
    maxSessions = _ref.maxSessions,
    providerType = _ref.providerType,
    requestTimeout = _ref.requestTimeout,
    transactionFee = _ref.transactionFee;

var POCKET_CONFIGURATION = new _pocketJs.Configuration(Number(maxDispatchers), Number(maxSessions), 0, Number(requestTimeout), undefined, undefined, Number(blockTime), undefined, undefined, false);
var POKT_DENOMINATIONS = {
  pokt: 0,
  upokt: 6
};
exports.POKT_DENOMINATIONS = POKT_DENOMINATIONS;

function getPocketDispatchers() {
  if (dispatchers === '') {
    return [];
  }

  return dispatchers.split(',').map(function (dispatcherUri) {
    return new URL(dispatcherUri);
  });
}

function getHttpRPCProvider() {
  if (!httpProviderNode || httpProviderNode === '') {
    throw new Error("Invalid HTTP Provider Node: ".concat(httpProviderNode));
  }

  return new _pocketJs.HttpRpcProvider(new URL(httpProviderNode));
}

function getRPCProvider() {
  if (providerType.toLowerCase() === 'http') {
    return getHttpRPCProvider();
  } else {
    return getHttpRPCProvider();
  }
}

function getNodes(_x) {
  return _getNodes.apply(this, arguments);
}

function _getNodes() {
  _getNodes = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(status) {
    var page, nodeList, perPage, pocketRpcProvider, pocketInstance, nodesResponse, totalPages, response;
    return _regenerator["default"].wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            page = 1;
            nodeList = [];
            perPage = 100;
            pocketRpcProvider = getRPCProvider();
            pocketInstance = new _pocketJs.Pocket(getPocketDispatchers(), undefined, POCKET_CONFIGURATION);
            _context.next = 7;
            return pocketInstance.rpc(pocketRpcProvider).query.getNodes(status, undefined, BigInt(0), undefined, page, perPage);

          case 7:
            nodesResponse = _context.sent;

            if (!(nodesResponse instanceof _pocketJs.RpcError)) {
              _context.next = 10;
              break;
            }

            return _context.abrupt("return", []);

          case 10:
            totalPages = nodesResponse.totalPages;
            nodesResponse.nodes.forEach(function (node) {
              nodeList.push(node);
            });
            page++;

          case 13:
            if (!(page <= totalPages)) {
              _context.next = 24;
              break;
            }

            _context.next = 16;
            return pocketInstance.rpc(pocketRpcProvider).query.getNodes(status, undefined, BigInt(0), undefined, page, perPage);

          case 16:
            response = _context.sent;
            // Increment page variable
            page++;

            if (!(response instanceof _pocketJs.RpcError)) {
              _context.next = 21;
              break;
            }

            page = totalPages;
            return _context.abrupt("return");

          case 21:
            response.nodes.forEach(function (node) {
              nodeList.push(node);
            });
            _context.next = 13;
            break;

          case 24:
            return _context.abrupt("return", nodesResponse.nodes);

          case 25:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  return _getNodes.apply(this, arguments);
}

function getApplications(_x2) {
  return _getApplications.apply(this, arguments);
}

function _getApplications() {
  _getApplications = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2(status) {
    var page, applicationList, pocketRpcProvider, perPage, pocketInstance, applicationsResponse, totalPages, response;
    return _regenerator["default"].wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            page = 1;
            applicationList = [];
            pocketRpcProvider = getRPCProvider();
            perPage = 100;
            pocketInstance = new _pocketJs.Pocket(getPocketDispatchers(), undefined, POCKET_CONFIGURATION);
            _context2.next = 7;
            return pocketInstance.rpc(pocketRpcProvider).query.getApps(status, BigInt(0), undefined, page, perPage);

          case 7:
            applicationsResponse = _context2.sent;

            if (!(applicationsResponse instanceof _pocketJs.RpcError)) {
              _context2.next = 10;
              break;
            }

            return _context2.abrupt("return", []);

          case 10:
            totalPages = applicationsResponse.totalPages;

          case 11:
            if (!(page <= totalPages)) {
              _context2.next = 22;
              break;
            }

            _context2.next = 14;
            return pocketInstance.rpc(pocketRpcProvider).query.getApps(status, BigInt(0), undefined, page, perPage);

          case 14:
            response = _context2.sent;
            page++;

            if (!(response instanceof _pocketJs.RpcError)) {
              _context2.next = 19;
              break;
            }

            page = totalPages;
            return _context2.abrupt("return");

          case 19:
            response.applications.forEach(function (app) {
              applicationList.push(app);
            });
            _context2.next = 11;
            break;

          case 22:
            return _context2.abrupt("return", applicationList);

          case 23:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));
  return _getApplications.apply(this, arguments);
}

function transferFromFreeTierFund(_x3, _x4) {
  return _transferFromFreeTierFund.apply(this, arguments);
}

function _transferFromFreeTierFund() {
  _transferFromFreeTierFund = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee3(amount, customerAddress) {
    var totalAmount, pocketInstance, pocketRpcProvider, rawTxResponse;
    return _regenerator["default"].wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            if (transactionFee) {
              _context3.next = 2;
              break;
            }

            throw new Error("Can't transfer from free tier: transaction fee missing");

          case 2:
            if (chainId) {
              _context3.next = 4;
              break;
            }

            throw new Error("Can't transfer from free tier: chainID missing");

          case 4:
            if (amount) {
              _context3.next = 6;
              break;
            }

            throw new Error("Can't transfer from free tier: no amount provided");

          case 6:
            if (customerAddress) {
              _context3.next = 8;
              break;
            }

            throw new Error("Can't transfer from free tier: no customer address provided");

          case 8:
            totalAmount = BigInt(Number(amount) + Number(transactionFee));

            if (totalAmount) {
              _context3.next = 11;
              break;
            }

            throw "Can't transfer from free tier: failed to calculate totalAmount";

          case 11:
            pocketInstance = new _pocketJs.Pocket(getPocketDispatchers(), undefined, POCKET_CONFIGURATION);
            pocketRpcProvider = getRPCProvider();
            pocketInstance.rpc(pocketRpcProvider);
            _context3.next = 16;
            return pocketInstance.withPrivateKey(freeTierFundAccount).send(freeTierFundAddress, customerAddress, totalAmount.toString()).submit(chainId, transactionFee);

          case 16:
            rawTxResponse = _context3.sent;

            if (!(0, _pocketJs.typeGuard)(rawTxResponse, _pocketJs.RpcError)) {
              _context3.next = 19;
              break;
            }

            throw new Error(rawTxResponse.message);

          case 19:
            return _context3.abrupt("return", rawTxResponse.hash);

          case 20:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));
  return _transferFromFreeTierFund.apply(this, arguments);
}

function createUnlockedAccount(_x5) {
  return _createUnlockedAccount.apply(this, arguments);
}

function _createUnlockedAccount() {
  _createUnlockedAccount = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee4(passphrase) {
    var pocketInstance, account, unlockedAccountOrError;
    return _regenerator["default"].wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            pocketInstance = new _pocketJs.Pocket(getPocketDispatchers(), undefined, POCKET_CONFIGURATION);
            _context4.next = 3;
            return pocketInstance.keybase.createAccount(passphrase);

          case 3:
            account = _context4.sent;
            _context4.next = 6;
            return pocketInstance.keybase.getUnlockedAccount(account.addressHex, passphrase);

          case 6:
            unlockedAccountOrError = _context4.sent;

            if (!(0, _pocketJs.typeGuard)(unlockedAccountOrError, Error)) {
              _context4.next = 11;
              break;
            }

            throw new Error(unlockedAccountOrError.message);

          case 11:
            if (!(0, _pocketJs.typeGuard)(unlockedAccountOrError, _pocketJs.UnlockedAccount)) {
              _context4.next = 15;
              break;
            }

            return _context4.abrupt("return", unlockedAccountOrError);

          case 15:
            throw new Error('Unknown error while creating an unlocked account');

          case 16:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4);
  }));
  return _createUnlockedAccount.apply(this, arguments);
}

function getBalance(_x6) {
  return _getBalance.apply(this, arguments);
}

function _getBalance() {
  _getBalance = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee5(addressHex) {
    var pocketInstance, pocketRpcProvider, applicationResponse;
    return _regenerator["default"].wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            pocketInstance = new _pocketJs.Pocket(getPocketDispatchers(), undefined, POCKET_CONFIGURATION);
            pocketRpcProvider = getRPCProvider();
            _context5.next = 4;
            return pocketInstance.rpc(pocketRpcProvider).query.getBalance(addressHex);

          case 4:
            applicationResponse = _context5.sent;
            return _context5.abrupt("return", applicationResponse);

          case 6:
          case "end":
            return _context5.stop();
        }
      }
    }, _callee5);
  }));
  return _getBalance.apply(this, arguments);
}

function getTX(_x7) {
  return _getTX.apply(this, arguments);
}

function _getTX() {
  _getTX = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee6(addressHex) {
    var pocketInstance, pocketRpcProvider, applicationResponse;
    return _regenerator["default"].wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            pocketInstance = new _pocketJs.Pocket(getPocketDispatchers(), undefined, POCKET_CONFIGURATION);
            pocketRpcProvider = getRPCProvider();
            _context6.next = 4;
            return pocketInstance.rpc(pocketRpcProvider).query.getTX(addressHex);

          case 4:
            applicationResponse = _context6.sent;
            return _context6.abrupt("return", applicationResponse);

          case 6:
          case "end":
            return _context6.stop();
        }
      }
    }, _callee6);
  }));
  return _getTX.apply(this, arguments);
}

function getAccount(_x8) {
  return _getAccount.apply(this, arguments);
}

function _getAccount() {
  _getAccount = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee7(addressHex) {
    var pocketInstance, pocketRpcProvider, applicationResponse;
    return _regenerator["default"].wrap(function _callee7$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:
            pocketInstance = new _pocketJs.Pocket(getPocketDispatchers(), undefined, POCKET_CONFIGURATION);
            pocketRpcProvider = getRPCProvider();
            _context7.next = 4;
            return pocketInstance.rpc(pocketRpcProvider).query.getAccount(addressHex);

          case 4:
            applicationResponse = _context7.sent;
            return _context7.abrupt("return", applicationResponse);

          case 6:
          case "end":
            return _context7.stop();
        }
      }
    }, _callee7);
  }));
  return _getAccount.apply(this, arguments);
}

function getApp(_x9) {
  return _getApp.apply(this, arguments);
}

function _getApp() {
  _getApp = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee8(addressHex) {
    var pocketInstance, pocketRpcProvider, applicationResponse;
    return _regenerator["default"].wrap(function _callee8$(_context8) {
      while (1) {
        switch (_context8.prev = _context8.next) {
          case 0:
            pocketInstance = new _pocketJs.Pocket(getPocketDispatchers(), undefined, POCKET_CONFIGURATION);
            pocketRpcProvider = getRPCProvider();
            _context8.next = 4;
            return pocketInstance.rpc(pocketRpcProvider).query.getApp(addressHex);

          case 4:
            applicationResponse = _context8.sent;
            return _context8.abrupt("return", applicationResponse);

          case 6:
          case "end":
            return _context8.stop();
        }
      }
    }, _callee8);
  }));
  return _getApp.apply(this, arguments);
}

function createAppStakeTx(_x10, _x11, _x12, _x13) {
  return _createAppStakeTx.apply(this, arguments);
}

function _createAppStakeTx() {
  _createAppStakeTx = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee9(passphrase, privateKey, chains, stakeAmount) {
    var pocketInstance, unlockedAccount, senderAccount, account;
    return _regenerator["default"].wrap(function _callee9$(_context9) {
      while (1) {
        switch (_context9.prev = _context9.next) {
          case 0:
            pocketInstance = new _pocketJs.Pocket(getPocketDispatchers(), undefined, POCKET_CONFIGURATION);
            _context9.next = 3;
            return pocketInstance.keybase.importAccount(privateKey, passphrase);

          case 3:
            unlockedAccount = _context9.sent;

            if (!(unlockedAccount instanceof Error)) {
              _context9.next = 6;
              break;
            }

            throw unlockedAccount;

          case 6:
            _context9.next = 8;
            return pocketInstance.withImportedAccount(unlockedAccount.addressHex, passphrase);

          case 8:
            senderAccount = _context9.sent;
            // @ts-ignore
            account = senderAccount.unlockedAccount;
            _context9.next = 12;
            return senderAccount.appStake(account.publicKey.toString('hex'), chains, stakeAmount.toString()).createTransaction(chainId, transactionFee);

          case 12:
            return _context9.abrupt("return", _context9.sent);

          case 13:
          case "end":
            return _context9.stop();
        }
      }
    }, _callee9);
  }));
  return _createAppStakeTx.apply(this, arguments);
}

function getPocketInstance() {
  return _getPocketInstance.apply(this, arguments);
}

function _getPocketInstance() {
  _getPocketInstance = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee10() {
    return _regenerator["default"].wrap(function _callee10$(_context10) {
      while (1) {
        switch (_context10.prev = _context10.next) {
          case 0:
            return _context10.abrupt("return", new _pocketJs.Pocket(getPocketDispatchers(), undefined, POCKET_CONFIGURATION));

          case 1:
          case "end":
            return _context10.stop();
        }
      }
    }, _callee10);
  }));
  return _getPocketInstance.apply(this, arguments);
}

function submitRawTransaction(_x14, _x15) {
  return _submitRawTransaction.apply(this, arguments);
}

function _submitRawTransaction() {
  _submitRawTransaction = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee11(fromAddress, rawTxBytes) {
    var pocketInstance, pocketRpcProvider, rawTxResponse;
    return _regenerator["default"].wrap(function _callee11$(_context11) {
      while (1) {
        switch (_context11.prev = _context11.next) {
          case 0:
            pocketInstance = new _pocketJs.Pocket(getPocketDispatchers(), undefined, POCKET_CONFIGURATION);
            pocketRpcProvider = getRPCProvider();
            _context11.next = 4;
            return pocketInstance.rpc(pocketRpcProvider).client.rawtx(fromAddress, rawTxBytes);

          case 4:
            rawTxResponse = _context11.sent;

            if (!(0, _pocketJs.typeGuard)(rawTxResponse, _pocketJs.RpcError)) {
              _context11.next = 7;
              break;
            }

            throw new Error(rawTxResponse.message);

          case 7:
            return _context11.abrupt("return", rawTxResponse.hash);

          case 8:
          case "end":
            return _context11.stop();
        }
      }
    }, _callee11);
  }));
  return _submitRawTransaction.apply(this, arguments);
}
//# sourceMappingURL=pocket.js.map