"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _mongoose = require("mongoose");

var _strongCryptor = require("strong-cryptor");

var _isEmail = _interopRequireDefault(require("validator/lib/isEmail"));

var _environment = _interopRequireDefault(require("../environment"));

var MIN_PRIVATE_KEY_LENGTH = 128;
var MIN_SECRET_KEY_LENGTH = 32;
var cryptoKey = (0, _environment["default"])('PERSISTENCE').dbEncryptionKey;
var encryptor = new _strongCryptor.Encryptor({
  key: cryptoKey
});
var decryptor = new _strongCryptor.Decryptor({
  key: cryptoKey
});
var applicationSchema = new _mongoose.Schema({
  chain: String,
  name: String,
  user: {
    type: _mongoose.Schema.Types.ObjectId,
    ref: 'User'
  },
  freeTier: Boolean,
  status: String,
  lastChangedStatusAt: Date,
  freeTierApplicationAccount: {
    address: String,
    publicKey: String,
    privateKey: String,
    passPhrase: String
  },
  gatewayAAT: {
    version: String,
    clientPublicKey: String,
    applicationPublicKey: String,
    applicationSignature: String
  },
  gatewaySettings: {
    secretKey: String,
    secretKeyRequired: Boolean,
    whitelistOrigins: [],
    whitelistUserAgents: []
  },
  notificationSettings: {
    signedUp: Boolean,
    quarter: Boolean,
    quarterLastSent: Date,
    half: Boolean,
    halfLastSent: Date,
    threeQuarters: Boolean,
    threeQuartersLastSent: Date,
    full: Boolean,
    fullLastSent: Date
  },
  createdAt: {
    type: Date,
    "default": new Date(Date.now())
  },
  updatedAt: {
    type: Date,
    "default": new Date(Date.now())
  }
}, {
  collection: 'Applications'
});

applicationSchema.statics.validateMetadata = function validateMetadata(_ref) {
  var name = _ref.name,
      owner = _ref.owner,
      user = _ref.user,
      contactEmail = _ref.contactEmail;

  if (!name || !owner || !user || !contactEmail) {
    return false;
  }

  return (0, _isEmail["default"])(contactEmail);
};

applicationSchema.statics.encryptSensitiveFields = function encryptSensitiveFields(privateKey, secretKey) {
  if (!(privateKey.length <= MIN_PRIVATE_KEY_LENGTH)) {
    throw new Error('Wrong private key length');
  }

  if (!(secretKey.length <= MIN_SECRET_KEY_LENGTH)) {
    throw new Error('Wrong secret key length');
  }

  var encryptedPrivateKey = encryptor.encrypt(privateKey);
  var encryptedSecretKey = encryptor.encrypt(secretKey);
  return {
    encryptedPrivateKey: encryptedPrivateKey,
    encryptedSecretKey: encryptedSecretKey
  };
};

applicationSchema.statics.decryptSensitiveFields = function decryptSensitiveFields(privateKey, secretKey) {
  if (!(privateKey.length <= MIN_PRIVATE_KEY_LENGTH)) {
    throw new Error('Wrong private key length');
  }

  if (!(secretKey.length <= MIN_SECRET_KEY_LENGTH)) {
    throw new Error('Wrong secret key length');
  }

  var encryptedPrivateKey = decryptor.decrypt(privateKey);
  var encryptedSecretKey = decryptor.decrypt(secretKey);
  return {
    encryptedPrivateKey: encryptedPrivateKey,
    encryptedSecretKey: encryptedSecretKey
  };
};

var ApplicationModel = (0, _mongoose.model)('Application', applicationSchema);
var _default = ApplicationModel;
exports["default"] = _default;
//# sourceMappingURL=Application.js.map