"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _mongoose = require("mongoose");

var _bcrypt = _interopRequireDefault(require("bcrypt"));

var _dotenv = _interopRequireDefault(require("dotenv"));

var _jsonwebtoken = _interopRequireDefault(require("jsonwebtoken"));

var _isEmail = _interopRequireDefault(require("validator/lib/isEmail"));

var _isStrongPassword = _interopRequireDefault(require("validator/lib/isStrongPassword"));

var _environment = _interopRequireDefault(require("../environment"));

_dotenv["default"].config();

var SALT_ROUNDS = 10;
var userSchema = new _mongoose.Schema({
  provider: String,
  email: String,
  username: String,
  password: String,
  lastLogin: String,
  validated: Boolean,
  v2: Boolean
}, {
  collection: 'Users'
});

userSchema.statics.validateEmail = function validateEmail(email) {
  return (0, _isEmail["default"])(email);
};

userSchema.statics.validatePassword = function validatePassword(password) {
  return (0, _isStrongPassword["default"])(password);
};

userSchema.statics.encryptPassword = function encryptPassword(password) {
  return _bcrypt["default"].hash(password, SALT_ROUNDS);
};

userSchema.statics.comparePassword = function comparePassword(plainPassword, userPassword) {
  return _bcrypt["default"].compare(plainPassword, userPassword);
};

userSchema.methods.generateVerificationToken = function generateVerificationToken() {
  var token = _jsonwebtoken["default"].sign({
    id: this._id
  }, (0, _environment["default"])('AUTH').privateSecret, {
    expiresIn: '10d',
    algorithm: 'RS256'
  });

  return token;
};

userSchema.methods.comparePassword = function comparePassword(password) {
  return _bcrypt["default"].compare(password, this.password);
};

var User = (0, _mongoose.model)('User', userSchema);
var _default = User;
exports["default"] = _default;
//# sourceMappingURL=User.js.map