"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.workers = exports.chains = exports.FREE_TIER_STAKE_AMOUNT = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _application = require("./application");

var _network = require("./network");

var _notifications = require("./notifications");

var _utils = require("./utils");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var TEST_ONLY_CHAINS = {
  POCKET_TESTNET: {
    ticker: 'POKT',
    id: '0002',
    limit: 3
  }
};
var TEST_CHAINS = {
  ETHEREUM_GOERLI_FULL: {
    ticker: 'ETH',
    id: '0026',
    limit: 2
  },
  ETHEREUM_RINKEBY_FULL: {
    ticker: 'ETH',
    id: '0025',
    limit: 3
  },
  ETHEREUM_ROPSTEN_FULL: {
    ticker: 'ETH',
    id: '0023',
    limit: 2
  }
};
var MAIN_CHAINS = {
  ETHEREUM_GOERLI_FULL: {
    ticker: "ETH",
    id: "0026",
    limit: 3
  },
  ETHEREUM_KOVAN_FULL: {
    ticker: "POA",
    id: "0024",
    limit: 2
  },
  ETHEREUM_MAINNET_ARCHIVAL: {
    ticker: 'ETH',
    id: '0022',
    limit: 1
  },
  FUSE_FULL: {
    ticker: 'FUSE',
    id: '0005',
    limit: 1
  },
  ETHEREUM_MAINNET_FULL: {
    ticker: 'ETH',
    id: '0021',
    limit: 3
  },
  ETHEREUM_RINKEBY_FULL: {
    ticker: 'ETH',
    id: '0025',
    limit: 2
  },
  ETHEREUM_XDAI_FULL: {
    ticker: 'POA',
    id: '0027',
    limit: 2
  },
  POCKET_MAINNET: {
    ticker: 'POKT',
    id: '0001',
    limit: 1
  }
};

function getChainsByEnvironment() {
  if (process.env.NODE_ENV === 'development') {
    return _objectSpread(_objectSpread({}, TEST_CHAINS), TEST_ONLY_CHAINS);
  }

  if (process.env.NODE_ENV === 'production') {
    return _objectSpread({}, MAIN_CHAINS);
  }
}

var FREE_TIER_STAKE_AMOUNT = 24950100000n;
exports.FREE_TIER_STAKE_AMOUNT = FREE_TIER_STAKE_AMOUNT;
var chains = getChainsByEnvironment();
/**
 * Holds the workers configuration.
 *
 * each type (category) of worker is color coded:
 * app-related workers (staking, unstaking, notifications) are colored green
 * network-related workers (counters) are colored yellow
 * misc workers are colored cyan
 */

exports.chains = chains;
var workers = [{
  name: 'App pool filler',
  color: 'green',
  workerFn: function workerFn(ctx) {
    return (0, _application.fillAppPool)(ctx);
  },
  recurrence: _utils.ONE_MINUTES
}, {
  name: 'App pool staker',
  color: 'green',
  workerFn: function workerFn(ctx) {
    return (0, _application.stakeAppPool)(ctx);
  },
  recurrence: _utils.FIVE_MINUTES
}, {
  name: 'Network stats counter',
  color: 'yellow',
  workerFn: function workerFn(ctx) {
    return (0, _network.getNetworkStatsCount)(ctx);
  },
  recurrence: _utils.SIXTY_MINUTES
}, {
  name: 'Nodes per chain counter',
  color: 'yellow',
  workerFn: function workerFn(ctx) {
    return (0, _network.getNodeCountForChains)(ctx);
  },
  recurrence: _utils.SIXTY_MINUTES
}, {
  name: 'Usage notification service',
  color: 'blue',
  workerFn: function workerFn(ctx) {
    return (0, _notifications.sendUsageNotifications)(ctx);
  },
  recurrence: _utils.FIFTEEN_MINUTES
}];
exports.workers = workers;
//# sourceMappingURL=config.js.map