"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getTimeDifferenceExceeded = getTimeDifferenceExceeded;
exports.sendUsageNotifications = sendUsageNotifications;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _taggedTemplateLiteral2 = _interopRequireDefault(require("@babel/runtime/helpers/taggedTemplateLiteral"));

var _dayjs = _interopRequireDefault(require("dayjs"));

var _utc = _interopRequireDefault(require("dayjs/plugin/utc"));

var _graphqlRequest = require("graphql-request");

var _MailgunService = _interopRequireDefault(require("../services/MailgunService"));

var _Application = _interopRequireDefault(require("../models/Application"));

var _User = _interopRequireDefault(require("../models/User"));

var _environment = _interopRequireDefault(require("../environment"));

var _templateObject;

function _createForOfIteratorHelper(o, allowArrayLike) { var it; if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = o[Symbol.iterator](); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var LAST_SENT_SUFFIX = 'LastSent';
var THRESHOLDS = new Map([['quarter', 25000], ['half', 50000], ['threeQuarters', 75000], ['full', 100000]]);
var DAILY_RELAYS_QUERY = (0, _graphqlRequest.gql)(_templateObject || (_templateObject = (0, _taggedTemplateLiteral2["default"])(["\n  query TOTAL_RELAYS_AND_AVG_LATENCY_QUERY($_eq: String, $_gte: timestamptz) {\n    relay_apps_daily(\n      where: { app_pub_key: { _eq: $_eq }, bucket: { _gte: $_gte } }\n      order_by: { bucket: desc }\n    ) {\n      total_relays\n      result\n      bucket\n    }\n  }\n"])));
var gqlClient = new _graphqlRequest.GraphQLClient((0, _environment["default"])('HASURA_URL'), {
  headers: {
    'x-hasura-admin-secret': (0, _environment["default"])('HASURA_SECRET')
  }
});

function fetchRelayData(_x) {
  return _fetchRelayData.apply(this, arguments);
}

function _fetchRelayData() {
  _fetchRelayData = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(publicKey) {
    var today, formattedTimestamp, res, _res$relay_apps_daily, rawDailyRelays;

    return _regenerator["default"].wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _dayjs["default"].extend(_utc["default"]);

            today = _dayjs["default"].utc();
            formattedTimestamp = "".concat(today.year(), "-0").concat(today.month() + 1, "-").concat(today.date(), "T00:00:00+00:00");
            _context.next = 5;
            return gqlClient.request(DAILY_RELAYS_QUERY, {
              _eq: publicKey,
              _gte: formattedTimestamp
            });

          case 5:
            res = _context.sent;
            _res$relay_apps_daily = res.relay_apps_daily, rawDailyRelays = _res$relay_apps_daily === void 0 ? [] : _res$relay_apps_daily;
            return _context.abrupt("return", rawDailyRelays);

          case 8:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  return _fetchRelayData.apply(this, arguments);
}

function calculateSentRelays(rawDailyRelays) {
  var dailyRelays = new Map();

  var _iterator = _createForOfIteratorHelper(rawDailyRelays),
      _step;

  try {
    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      var _step$value = _step.value,
          bucket = _step$value.bucket,
          dailyRelayCount = _step$value.total_relays;

      if (!dailyRelays.has(bucket)) {
        dailyRelays.set(bucket, dailyRelayCount);
      } else {
        var currentCount = dailyRelays.get(bucket);
        dailyRelays.set(bucket, Number(currentCount) + Number(dailyRelayCount));
      }
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }

  var servedRelays = 0;

  var _iterator2 = _createForOfIteratorHelper(dailyRelays.entries()),
      _step2;

  try {
    for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
      var _step2$value = (0, _slicedToArray2["default"])(_step2.value, 2),
          _dailyRelayCount = _step2$value[1];

      servedRelays += _dailyRelayCount;
    }
  } catch (err) {
    _iterator2.e(err);
  } finally {
    _iterator2.f();
  }

  return servedRelays;
}

function calculateExceededThreshold(servedRelays) {
  var highestThresholdExceeded = 0;
  var thresholdKey = '';

  var _iterator3 = _createForOfIteratorHelper(THRESHOLDS.entries()),
      _step3;

  try {
    for (_iterator3.s(); !(_step3 = _iterator3.n()).done;) {
      var _step3$value = (0, _slicedToArray2["default"])(_step3.value, 2),
          key = _step3$value[0],
          threshold = _step3$value[1];

      if (servedRelays > threshold) {
        highestThresholdExceeded = threshold;
        thresholdKey = key;
      }
    }
  } catch (err) {
    _iterator3.e(err);
  } finally {
    _iterator3.f();
  }

  return [thresholdKey, highestThresholdExceeded];
}

function getTimeDifferenceExceeded(notificationSettings, thresholdKey) {
  var _notificationSettings;

  if (!("".concat(thresholdKey).concat(LAST_SENT_SUFFIX) in notificationSettings)) {
    return false;
  } // Edge case: if days are different no matter time elapsed we should be able to send the email


  var sent = (0, _dayjs["default"])(notificationSettings["".concat(thresholdKey).concat(LAST_SENT_SUFFIX)]);
  var now = (0, _dayjs["default"])();

  if (sent.date() !== now.date()) {
    return true;
  }

  return (0, _dayjs["default"])().diff((_notificationSettings = notificationSettings["".concat(thresholdKey).concat(LAST_SENT_SUFFIX)]) !== null && _notificationSettings !== void 0 ? _notificationSettings : (0, _dayjs["default"])(), 'day') > 0;
}

function sendUsageNotifications(_x2) {
  return _sendUsageNotifications.apply(this, arguments);
}

function _sendUsageNotifications() {
  _sendUsageNotifications = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2(ctx) {
    var applications, _iterator4, _step4, application, freeTierApplicationAccount, notificationSettings, userId, appName, appId, publicKey, rawDailyRelayData, servedRelays, _calculateExceededThr, _calculateExceededThr2, thresholdKey, highestThresholdExceeded, shouldSendNotification, user, _user$email, userEmail, emailService, totalUsage;

    return _regenerator["default"].wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return _Application["default"].find({
              status: {
                $exists: true
              },
              notificationSettings: {
                $exists: true
              }
            });

          case 2:
            applications = _context2.sent;
            _iterator4 = _createForOfIteratorHelper(applications);
            _context2.prev = 4;

            _iterator4.s();

          case 6:
            if ((_step4 = _iterator4.n()).done) {
              _context2.next = 31;
              break;
            }

            application = _step4.value;
            freeTierApplicationAccount = application.freeTierApplicationAccount, notificationSettings = application.notificationSettings, userId = application.user, appName = application.name, appId = application._id;
            publicKey = freeTierApplicationAccount.publicKey;
            _context2.next = 12;
            return fetchRelayData(publicKey);

          case 12:
            rawDailyRelayData = _context2.sent;
            servedRelays = calculateSentRelays(rawDailyRelayData);
            _calculateExceededThr = calculateExceededThreshold(servedRelays), _calculateExceededThr2 = (0, _slicedToArray2["default"])(_calculateExceededThr, 2), thresholdKey = _calculateExceededThr2[0], highestThresholdExceeded = _calculateExceededThr2[1];
            shouldSendNotification = highestThresholdExceeded > 0 && notificationSettings[thresholdKey] && getTimeDifferenceExceeded(notificationSettings, thresholdKey);

            if (!shouldSendNotification) {
              _context2.next = 29;
              break;
            }

            _context2.next = 19;
            return _User["default"].findById(userId);

          case 19:
            user = _context2.sent;

            if (!user) {
              ctx.logger.warn("NOTICE(Notifications): Orphaned app ".concat(appId, ") from user ").concat(userId, " getting usage."));
            }

            _user$email = user.email, userEmail = _user$email === void 0 ? '' : _user$email;
            emailService = new _MailgunService["default"]();
            totalUsage = (servedRelays / THRESHOLDS.get('full') * 100).toFixed(2);
            ctx.logger.log("Notifying app ".concat(appName, " (ID: ").concat(appId, ") from user ").concat(userId, " of ").concat(totalUsage, "% usage"));
            emailService.send({
              templateData: {
                app_name: appName,
                app_id: appId.toString(),
                usage: "".concat(totalUsage, "%")
              },
              templateName: 'NotificationThresholdHit',
              toEmail: userEmail
            });
            application.notificationSettings["".concat(thresholdKey).concat(LAST_SENT_SUFFIX)] = new Date(Date.now());
            _context2.next = 29;
            return application.save();

          case 29:
            _context2.next = 6;
            break;

          case 31:
            _context2.next = 36;
            break;

          case 33:
            _context2.prev = 33;
            _context2.t0 = _context2["catch"](4);

            _iterator4.e(_context2.t0);

          case 36:
            _context2.prev = 36;

            _iterator4.f();

            return _context2.finish(36);

          case 39:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2, null, [[4, 33, 36, 39]]);
  }));
  return _sendUsageNotifications.apply(this, arguments);
}
//# sourceMappingURL=notifications.js.map